from rips.pdmobject import PdmObjectBase
class CellFilterCollection(PdmObjectBase):
    """
    Attributes:
        active (bool): Active
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.active = True
        PdmObjectBase.__init__(self, pb2_object, channel)
        if CellFilterCollection.__custom_init__ is not None:
            CellFilterCollection.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class CurveIntersection(PdmObjectBase):
    """
    Attributes:
        name (str): Name
        points (List of str): Points
        simulation_well (str): Simulation Well
        type (str): One of [CS_WELL_PATH, CS_SIMULATION_WELL, CS_POLYLINE, CS_AZIMUTHLINE]
        well_path (str): Well Path        
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.name = "Intersection Name"
        self.points = []
        self.simulation_well = ""
        self.type = "CS_POLYLINE"
        self.well_path = ""
        PdmObjectBase.__init__(self, pb2_object, channel)
        if CurveIntersection.__custom_init__ is not None:
            CurveIntersection.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def geometry(self, geometry_type="FULL_3D"):
        """
        

        Arguments:
            geometry_type (str): One of [FULL_3D, PROJECTED_TO_PLANE]
        Returns:
            TriangleGeometry
        """
        return self._call_pdm_method("geometry", geometry_type=geometry_type)


    def geometry_result(self, geometry_type="FULL_3D"):
        """
        

        Arguments:
            geometry_type (str): One of [FULL_3D, PROJECTED_TO_PLANE]
        Returns:
            DataContainerFloat
        """
        return self._call_pdm_method("geometryResult", geometry_type=geometry_type)


class DataContainerFloat(PdmObjectBase):
    """
    Attributes:
        values (List of float): Float Values
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.values = []
        PdmObjectBase.__init__(self, pb2_object, channel)
        if DataContainerFloat.__custom_init__ is not None:
            DataContainerFloat.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class DataContainerString(PdmObjectBase):
    """
    Attributes:
        values (List of str): String Values
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.values = []
        PdmObjectBase.__init__(self, pb2_object, channel)
        if DataContainerString.__custom_init__ is not None:
            DataContainerString.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class DataContainerTime(PdmObjectBase):
    """
    Attributes:
        values (List of time): Time Values
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.values = []
        PdmObjectBase.__init__(self, pb2_object, channel)
        if DataContainerTime.__custom_init__ is not None:
            DataContainerTime.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class Case(PdmObjectBase):
    """
    The ResInsight base class for Cases

    Attributes:
        file_path (str): Case File Name
        id (int): Case ID
        name (str): Case Name
        name_setting (str): One of [FULL_CASE_NAME, SHORT_CASE_NAME, CUSTOM_NAME]
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.file_path = ""
        self.id = -1
        self.name = ""
        self.name_setting = "FULL_CASE_NAME"
        PdmObjectBase.__init__(self, pb2_object, channel)
        if Case.__custom_init__ is not None:
            Case.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class Reservoir(Case):
    """
    Abstract base class for Eclipse Cases

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        Case.__init__(self, pb2_object, channel)
        if Reservoir.__custom_init__ is not None:
            Reservoir.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def import_properties(self, file_names=[]):
        """
        Import Properties

        Arguments:
            file_names (List of str): 
        Returns:
            
        """
        return self._call_pdm_method("import_properties", file_names=file_names)


    def views(self):
        """All Eclipse Views in the case

        Returns:
             List of EclipseView
        """
        return self.children("Views", EclipseView)


class EclipseCase(Reservoir):
    """
    The Regular Eclipse Results Case

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        Reservoir.__init__(self, pb2_object, channel)
        if EclipseCase.__custom_init__ is not None:
            EclipseCase.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class ElasticProperties(PdmObjectBase):
    """
    Attributes:
        file_path (str): File Path
        properties_table (str): Properties Table
        show_scaled_properties (bool): Show Scaled Properties
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.file_path = ""
        self.properties_table = ""
        self.show_scaled_properties = True
        PdmObjectBase.__init__(self, pb2_object, channel)
        if ElasticProperties.__custom_init__ is not None:
            ElasticProperties.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def add_property_scaling(self, formation="", facies="", property="", scale=1):
        """
        Add Elastic Property Scaling

        Arguments:
            formation (str): Formation
            facies (str): Facies
            property (str): Property
            scale (float): Scale
        Returns:
            ElasticPropertyScaling
        """
        return self._call_pdm_method("AddPropertyScaling", formation=formation, facies=facies, property=property, scale=scale)


    def property_scaling_collection(self):
        """PropertyScalingCollection

        Returns:
             ElasticPropertyScalingCollection
        """
        children = self.children("PropertyScalingCollection", ElasticPropertyScalingCollection)
        return children[0] if len(children) > 0 else None


class NamedObject(PdmObjectBase):
    """
    Attributes:
        name (str): Name
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.name = ""
        PdmObjectBase.__init__(self, pb2_object, channel)
        if NamedObject.__custom_init__ is not None:
            NamedObject.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class CheckableNamedObject(NamedObject):
    """
    Attributes:
        is_checked (bool): Active
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.is_checked = True
        NamedObject.__init__(self, pb2_object, channel)
        if CheckableNamedObject.__custom_init__ is not None:
            CheckableNamedObject.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class ElasticPropertyScaling(CheckableNamedObject):
    """
    Attributes:
        facies (str): Facies
        formation (str): Formation
        property (str): One of [UNDEFINED, FACIES, LAYERS, POROSITY, PERMEABILITY_X, PERMEABILITY_Z, INITIAL_PRESSURE, PRESSURE, STRESS, INITIAL_STRESS, STRESS_GRADIENT, YOUNGS_MODULUS, POISSONS_RATIO, K_IC, PROPPANT_EMBEDMENT, BIOT_COEFFICIENT, K0, FLUID_LOSS_COEFFICIENT, SPURT_LOSS, TEMPERATURE, RELATIVE_PERMEABILITY_FACTOR, PORO_ELASTIC_CONSTANT, THERMAL_EXPANSION_COEFFICIENT, IMMOBILE_FLUID_SATURATION, NET_TO_GROSS, POROSITY_UNSCALED, EQLNUM, PRESSURE_GRADIENT, FORMATIONS]
        scale (float): Scale
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.facies = ""
        self.formation = ""
        self.property = "YOUNGS_MODULUS"
        self.scale = 1
        CheckableNamedObject.__init__(self, pb2_object, channel)
        if ElasticPropertyScaling.__custom_init__ is not None:
            ElasticPropertyScaling.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class ElasticPropertyScalingCollection(PdmObjectBase):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        PdmObjectBase.__init__(self, pb2_object, channel)
        if ElasticPropertyScalingCollection.__custom_init__ is not None:
            ElasticPropertyScalingCollection.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def elastic_property_scalings(self):
        """Elastic Property Scalings

        Returns:
             List of ElasticPropertyScaling
        """
        return self.children("ElasticPropertyScalings", ElasticPropertyScaling)


class SurfaceInterface(PdmObjectBase):
    """
    Attributes:
        depth_offset (float): Depth Offset
        surface_user_decription (str): Name
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.depth_offset = 0
        self.surface_user_decription = ""
        PdmObjectBase.__init__(self, pb2_object, channel)
        if SurfaceInterface.__custom_init__ is not None:
            SurfaceInterface.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def export_to_file(self, file_name=""):
        """
        Export a surface to file

        Arguments:
            file_name (str): Filename to export surface to
        Returns:
            DataContainerString
        """
        return self._call_pdm_method("ExportToFile", file_name=file_name)


class EnsembleStatisticsSurface(SurfaceInterface):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        SurfaceInterface.__init__(self, pb2_object, channel)
        if EnsembleStatisticsSurface.__custom_init__ is not None:
            EnsembleStatisticsSurface.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class SurfaceCollection(PdmObjectBase):
    """
    Attributes:
        surface_user_decription (str): Name
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.surface_user_decription = "Surfaces"
        PdmObjectBase.__init__(self, pb2_object, channel)
        if SurfaceCollection.__custom_init__ is not None:
            SurfaceCollection.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def add_folder(self, folder_name="Surfaces"):
        """
        Add a new surface folder

        Arguments:
            folder_name (str): New surface folder name
        Returns:
            SurfaceCollection
        """
        return self._call_pdm_method("AddFolder", folder_name=folder_name)


    def import_surface(self, file_name=""):
        """
        Import a new surface from file

        Arguments:
            file_name (str): Filename to import surface from
        Returns:
            Surface
        """
        return self._call_pdm_method("ImportSurface", file_name=file_name)


    def new_surface(self, case="", k_index=0):
        """
        Create a new surface

        Arguments:
            case (Case): 
            k_index (int): 
        Returns:
            GridCaseSurface
        """
        return self._call_pdm_method("NewSurface", case=case, k_index=k_index)


    def sub_collections(self):
        """Surfaces

        Returns:
             List of SurfaceCollection
        """
        return self.children("SubCollections", SurfaceCollection)


    def surfaces_field(self):
        """Surfaces

        Returns:
             List of SurfaceInterface
        """
        return self.children("SurfacesField", SurfaceInterface)


class EnsembleSurface(SurfaceCollection):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        SurfaceCollection.__init__(self, pb2_object, channel)
        if EnsembleSurface.__custom_init__ is not None:
            EnsembleSurface.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class EnsembleWellLogs(NamedObject):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        NamedObject.__init__(self, pb2_object, channel)
        if EnsembleWellLogs.__custom_init__ is not None:
            EnsembleWellLogs.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class FaciesInitialPressureConfig(PdmObjectBase):
    """
    Attributes:
        facies_name (str): Facies
        facies_value (int): Value
        fraction (float): Δ Pressure Fraction
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.facies_name = ""
        self.facies_value = -1
        self.fraction = 0
        PdmObjectBase.__init__(self, pb2_object, channel)
        if FaciesInitialPressureConfig.__custom_init__ is not None:
            FaciesInitialPressureConfig.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class FaciesProperties(PdmObjectBase):
    """
    Attributes:
        color_legend (str): Colors
        file_path (str): File Path
        properties_table (str): Properties Table
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.color_legend = ""
        self.file_path = ""
        self.properties_table = ""
        PdmObjectBase.__init__(self, pb2_object, channel)
        if FaciesProperties.__custom_init__ is not None:
            FaciesProperties.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def facies_definition(self):
        """

        Returns:
             EclipseResult
        """
        children = self.children("FaciesDefinition", EclipseResult)
        return children[0] if len(children) > 0 else None


class SummaryCase(PdmObjectBase):
    """
    The Base Class for all Summary Cases

    Attributes:
        auto_shorty_name (bool): Use Auto Display Name
        id (int): Case ID
        name_setting (str): One of [FULL_CASE_NAME, SHORT_CASE_NAME, CUSTOM_NAME]
        short_name (str): Display Name
        show_sub_nodes_in_tree (bool): Show Summary Data Sub-Tree
        summary_header_filename (str): Summary Header File
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.auto_shorty_name = False
        self.id = -1
        self.name_setting = "FULL_CASE_NAME"
        self.short_name = ""
        self.show_sub_nodes_in_tree = False
        self.summary_header_filename = ""
        PdmObjectBase.__init__(self, pb2_object, channel)
        if SummaryCase.__custom_init__ is not None:
            SummaryCase.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def available_addresses(self, ):
        """
        

        Arguments:
            
        Returns:
            DataContainerString
        """
        return self._call_pdm_method("availableAddresses")


    def available_time_steps(self, ):
        """
        

        Arguments:
            
        Returns:
            DataContainerTime
        """
        return self._call_pdm_method("availableTimeSteps")


    def resample_values(self, address="", resampling_period=""):
        """
        

        Arguments:
            address (str): Formatted address specifying the summary vector
            resampling_period (str): Resampling Period
        Returns:
            ResampleData
        """
        return self._call_pdm_method("resampleValues", address=address, resampling_period=resampling_period)


    def set_summary_values(self, address="", unit="", values=[]):
        """
        

        Arguments:
            address (str): Formatted address specifying the summary vector
            unit (str): Unit
            values (List of float): Values
        Returns:
            
        """
        return self._call_pdm_method("setSummaryValues", address=address, unit=unit, values=values)


    def summary_vector_values(self, address=""):
        """
        Get all values for a summary vector

        Arguments:
            address (str): Formatted address specifying the summary vector
        Returns:
            DataContainerFloat
        """
        return self._call_pdm_method("summaryVectorValues", address=address)


class FileSummaryCase(SummaryCase):
    """
    A Summary Case based on SMSPEC files

    Attributes:
        include_restart_files (bool): Include Restart Files
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.include_restart_files = False
        SummaryCase.__init__(self, pb2_object, channel)
        if FileSummaryCase.__custom_init__ is not None:
            FileSummaryCase.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class FractureTemplateCollection(PdmObjectBase):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        PdmObjectBase.__init__(self, pb2_object, channel)
        if FractureTemplateCollection.__custom_init__ is not None:
            FractureTemplateCollection.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def append_fracture_template(self, file_path=""):
        """
        Create a new StimPlan Fracture Template

        Arguments:
            file_path (str): File Path to StimPlan Countour File
        Returns:
            StimPlanFractureTemplate
        """
        return self._call_pdm_method("AppendFractureTemplate", file_path=file_path)


    def append_thermal_fracture_template(self, file_path=""):
        """
        Create a new Thermal Fracture Template

        Arguments:
            file_path (str): File Path to Thermal Fracture CSV File
        Returns:
            ThermalFractureTemplate
        """
        return self._call_pdm_method("AppendThermalFractureTemplate", file_path=file_path)


class GeoMechPart(CheckableNamedObject):
    """
    Attributes:
        part_id (int): Part Id
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.part_id = -1
        CheckableNamedObject.__init__(self, pb2_object, channel)
        if GeoMechPart.__custom_init__ is not None:
            GeoMechPart.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class GeoMechPartCollection(PdmObjectBase):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        PdmObjectBase.__init__(self, pb2_object, channel)
        if GeoMechPartCollection.__custom_init__ is not None:
            GeoMechPartCollection.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def parts(self):
        """Parts

        Returns:
             List of GeoMechPart
        """
        return self.children("Parts", GeoMechPart)


class ViewWindow(PdmObjectBase):
    """
    The Base Class for all Views and Plots in ResInsight

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        PdmObjectBase.__init__(self, pb2_object, channel)
        if ViewWindow.__custom_init__ is not None:
            ViewWindow.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class View(ViewWindow):
    """
    Attributes:
        background_color (str): Background
        current_time_step (int): Current Time Step
        disable_lighting (bool): Disable Results Lighting
        grid_z_scale (float): Z Scale
        id (int): View ID
        perspective_projection (bool): Perspective Projection
        show_grid_box (bool): Show Grid Box
        show_z_scale (bool): Show Z Scale Label
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.background_color = "#b0c4de"
        self.current_time_step = 0
        self.disable_lighting = False
        self.grid_z_scale = 5
        self.id = -1
        self.perspective_projection = True
        self.show_grid_box = True
        self.show_z_scale = True
        ViewWindow.__init__(self, pb2_object, channel)
        if View.__custom_init__ is not None:
            View.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class GeoMechView(View):
    """
    The Geomechanical 3d View

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        View.__init__(self, pb2_object, channel)
        if GeoMechView.__custom_init__ is not None:
            GeoMechView.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class GridCaseSurface(SurfaceInterface):
    """
    Attributes:
        include_inactive_cells (bool): Include Inactive Cells
        slice_index (int): Slice Index (K)
        source_case (str): Source Case
        watertight (bool): Watertight Surface (fill gaps)
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.include_inactive_cells = False
        self.slice_index = 1
        self.source_case = ""
        self.watertight = False
        SurfaceInterface.__init__(self, pb2_object, channel)
        if GridCaseSurface.__custom_init__ is not None:
            GridCaseSurface.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class GridSummaryCase(SummaryCase):
    """
    A Summary Case based on extracting grid data.

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        SummaryCase.__init__(self, pb2_object, channel)
        if GridSummaryCase.__custom_init__ is not None:
            GridSummaryCase.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class IntersectionCollection(PdmObjectBase):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        PdmObjectBase.__init__(self, pb2_object, channel)
        if IntersectionCollection.__custom_init__ is not None:
            IntersectionCollection.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class WellPath(PdmObjectBase):
    """
    A ResInsight Well Path

    Attributes:
        name (str): Name
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.name = ""
        PdmObjectBase.__init__(self, pb2_object, channel)
        if WellPath.__custom_init__ is not None:
            WellPath.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def add_fracture(self, measured_depth=0, stim_plan_fracture_template="", align_dip=False, eclipse_case=""):
        """
        Add StimPlan Fracture

        Arguments:
            measured_depth (float): 
            stim_plan_fracture_template (StimPlanFractureTemplate): StimPlan Fracture Template
            align_dip (bool): 
            eclipse_case (RimReservoir): Eclipse Case
        Returns:
            WellPathFracture
        """
        return self._call_pdm_method("AddFracture", measured_depth=measured_depth, stim_plan_fracture_template=stim_plan_fracture_template, align_dip=align_dip, eclipse_case=eclipse_case)


    def add_thermal_fracture(self, measured_depth=0, fracture_template="", place_using_template_data=True):
        """
        Add Thermal Fracture

        Arguments:
            measured_depth (float): 
            fracture_template (ThermalFractureTemplate): Thermal Fracture Template
            place_using_template_data (bool): 
        Returns:
            WellPathFracture
        """
        return self._call_pdm_method("AddThermalFracture", measured_depth=measured_depth, fracture_template=fracture_template, place_using_template_data=place_using_template_data)


    def append_perforation_interval(self, start_md=0, end_md=0, diameter=0, skin_factor=0):
        """
        Append Perforation Interval

        Arguments:
            start_md (float): Start Measured Depth
            end_md (float): End Measured Depth
            diameter (float): Diameter
            skin_factor (float): Skin Factor
        Returns:
            Perforation
        """
        return self._call_pdm_method("AppendPerforationInterval", start_md=start_md, end_md=end_md, diameter=diameter, skin_factor=skin_factor)


class ModeledWellPath(WellPath):
    """
    A Well Path created interactively in ResInsight

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        WellPath.__init__(self, pb2_object, channel)
        if ModeledWellPath.__custom_init__ is not None:
            ModeledWellPath.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def append_lateral(self, tie_in_depth=0, lateral_name=""):
        """
        Append Well Path Lateral

        Arguments:
            tie_in_depth (float): Measured Depth on the Parent Well Path
            lateral_name (str): Lateral Name
        Returns:
            ModeledWellPath
        """
        return self._call_pdm_method("AppendLateral", tie_in_depth=tie_in_depth, lateral_name=lateral_name)


    def well_path_geometry(self):
        """Trajectory

        Returns:
             WellPathGeometry
        """
        children = self.children("WellPathGeometry", WellPathGeometry)
        return children[0] if len(children) > 0 else None


class NonNetLayers(PdmObjectBase):
    """
    Attributes:
        cutoff (float): Cutoff
        facies (str): Facies
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.cutoff = 0
        self.facies = ""
        PdmObjectBase.__init__(self, pb2_object, channel)
        if NonNetLayers.__custom_init__ is not None:
            NonNetLayers.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def facies_definition(self):
        """

        Returns:
             EclipseResult
        """
        children = self.children("FaciesDefinition", EclipseResult)
        return children[0] if len(children) > 0 else None


class PressureTable(PdmObjectBase):
    """
    Attributes:
        pressure_date (str): Pressure Date
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.pressure_date = ""
        PdmObjectBase.__init__(self, pb2_object, channel)
        if PressureTable.__custom_init__ is not None:
            PressureTable.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def items(self):
        """Pressure Table Items

        Returns:
             List of PressureTableItem
        """
        return self.children("Items", PressureTableItem)


class PressureTableItem(PdmObjectBase):
    """
    Attributes:
        depth (float): Depth TVDMSL [m]
        initial_pressure (float): Initial Pressure [Bar]
        pressure (float): Pressure [Bar]
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.depth = 0
        self.initial_pressure = 0
        self.pressure = 0
        PdmObjectBase.__init__(self, pb2_object, channel)
        if PressureTableItem.__custom_init__ is not None:
            PressureTableItem.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class GeoMechCase(Case):
    """
    The Abaqus Based GeoMech Case

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        Case.__init__(self, pb2_object, channel)
        if GeoMechCase.__custom_init__ is not None:
            GeoMechCase.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def views(self):
        """All GeoMech Views in the Case

        Returns:
             List of GeoMechView
        """
        return self.children("Views", GeoMechView)


class Project(PdmObjectBase):
    """
    The ResInsight Project

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        PdmObjectBase.__init__(self, pb2_object, channel)
        if Project.__custom_init__ is not None:
            Project.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def import_summary_case(self, file_name=""):
        """
        Import Summary Case

        Arguments:
            file_name (str): 
        Returns:
            FileSummaryCase
        """
        return self._call_pdm_method("importSummaryCase", file_name=file_name)


    def summary_case(self, case_id=-1):
        """
        Find Summary Case

        Arguments:
            case_id (int): 
        Returns:
            FileSummaryCase
        """
        return self._call_pdm_method("summaryCase", case_id=case_id)


    def surface_folder(self, folder_name=""):
        """
        Get Surface Folder

        Arguments:
            folder_name (str): 
        Returns:
            SurfaceCollection
        """
        return self._call_pdm_method("surfaceFolder", folder_name=folder_name)


class ResampleData(PdmObjectBase):
    """
    Attributes:
        time_steps (List of time): Time Steps
        values (List of float): Values
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.time_steps = []
        self.values = []
        PdmObjectBase.__init__(self, pb2_object, channel)
        if ResampleData.__custom_init__ is not None:
            ResampleData.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class EclipseView(View):
    """
    The Eclipse 3d Reservoir View

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        View.__init__(self, pb2_object, channel)
        if EclipseView.__custom_init__ is not None:
            EclipseView.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def cell_result(self):
        """Cell Result

        Returns:
             CellColors
        """
        children = self.children("CellResult", CellColors)
        return children[0] if len(children) > 0 else None


    def cell_result_data(self):
        """Current Eclipse Cell Result

        Returns:
             str
        """
        return self._call_get_method("CellResultData")


    def set_cell_result_data(self, values):
        """Set Current Eclipse Cell Result

        Arguments:
            values (str): data
        """
        self._call_set_method("CellResultData", values)


class EclipseResult(PdmObjectBase):
    """
    An eclipse result definition

    Attributes:
        flow_tracer_selection_mode (str): One of [FLOW_TR_INJ_AND_PROD, FLOW_TR_PRODUCERS, FLOW_TR_INJECTORS, FLOW_TR_BY_SELECTION]
        phase_selection (str): One of [PHASE_ALL, PHASE_OIL, PHASE_GAS, PHASE_WAT]
        porosity_model_type (str): One of [MATRIX_MODEL, FRACTURE_MODEL]
        result_type (str): One of [DYNAMIC_NATIVE, STATIC_NATIVE, SOURSIMRL, GENERATED, INPUT_PROPERTY, FORMATION_NAMES, ALLAN_DIAGRAMS, FLOW_DIAGNOSTICS, INJECTION_FLOODING]
        result_variable (str): Variable
        selected_injector_tracers (List of str): Injector Tracers
        selected_producer_tracers (List of str): Producer Tracers
        selected_souring_tracers (List of str): Tracers
        show_only_visible_categories_in_legend (bool): Show Only Visible Categories In Legend
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.flow_tracer_selection_mode = "FLOW_TR_INJ_AND_PROD"
        self.phase_selection = "PHASE_ALL"
        self.porosity_model_type = "MATRIX_MODEL"
        self.result_type = "DYNAMIC_NATIVE"
        self.result_variable = "None"
        self.selected_injector_tracers = []
        self.selected_producer_tracers = []
        self.selected_souring_tracers = []
        self.show_only_visible_categories_in_legend = True
        PdmObjectBase.__init__(self, pb2_object, channel)
        if EclipseResult.__custom_init__ is not None:
            EclipseResult.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class CellColors(EclipseResult):
    """
    Eclipse Cell Colors class

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        EclipseResult.__init__(self, pb2_object, channel)
        if CellColors.__custom_init__ is not None:
            CellColors.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class CommandRouter(PdmObjectBase):
    """
    The CommandRouter is used to call code independent to the project

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        PdmObjectBase.__init__(self, pb2_object, channel)
        if CommandRouter.__custom_init__ is not None:
            CommandRouter.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def extract_surfaces(self, grid_model_filename="", layers=[], minimum_i=-1, maximum_i=-1, minimum_j=-1, maximum_j=-1):
        """
        Extract Layer Surface

        Arguments:
            grid_model_filename (str): 
            layers (List of int): 
            minimum_i (int): 
            maximum_i (int): 
            minimum_j (int): 
            maximum_j (int): 
        Returns:
            
        """
        return self._call_pdm_method("ExtractSurfaces", grid_model_filename=grid_model_filename, layers=layers, minimum_i=minimum_i, maximum_i=maximum_i, minimum_j=minimum_j, maximum_j=maximum_j)


class EclipseContourMap(EclipseView):
    """
    A contour map for Eclipse cases

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        EclipseView.__init__(self, pb2_object, channel)
        if EclipseContourMap.__custom_init__ is not None:
            EclipseContourMap.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class GeoMechContourMap(GeoMechView):
    """
    A contour map for GeoMech cases

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        GeoMechView.__init__(self, pb2_object, channel)
        if GeoMechContourMap.__custom_init__ is not None:
            GeoMechContourMap.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class GridCaseGroup(PdmObjectBase):
    """
    A statistics case group

    Attributes:
        group_id (int): Case Group ID
        user_description (str): Name
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.group_id = -1
        self.user_description = "Grid Case Group"
        PdmObjectBase.__init__(self, pb2_object, channel)
        if GridCaseGroup.__custom_init__ is not None:
            GridCaseGroup.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class MudWeightWindowParameters(PdmObjectBase):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        PdmObjectBase.__init__(self, pb2_object, channel)
        if MudWeightWindowParameters.__custom_init__ is not None:
            MudWeightWindowParameters.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class FractureTemplate(PdmObjectBase):
    """
    Attributes:
        azimuth_angle (float): Azimuth Angle
        conductivity_factor (float): Conductivity
        conductivity_type (str): One of [InfiniteConductivity, FiniteConductivity]
        d_factor_scale_factor (float): D-factor
        height_scale_factor (float): Height
        orientation (str): One of [Azimuth, Longitudinal, Transverse]
        perforation_length (float): Perforation Length
        user_defined_perforation_length (bool): User-defined Perforation Length
        user_description (str): Name
        width_scale_factor (float): Half Length
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.azimuth_angle = 0
        self.conductivity_factor = 1
        self.conductivity_type = "FiniteConductivity"
        self.d_factor_scale_factor = 1
        self.height_scale_factor = 1
        self.orientation = "Transverse"
        self.perforation_length = 1
        self.user_defined_perforation_length = False
        self.user_description = "Fracture Template"
        self.width_scale_factor = 1
        PdmObjectBase.__init__(self, pb2_object, channel)
        if FractureTemplate.__custom_init__ is not None:
            FractureTemplate.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def set_scale_factors(self, half_length=1, height=1, d_factor=1, conductivity=1):
        """
        Set Fracture Template Scale Factors.

        Arguments:
            half_length (float): 
            height (float): 
            d_factor (float): 
            conductivity (float): 
        Returns:
            
        """
        return self._call_pdm_method("SetScaleFactors", half_length=half_length, height=height, d_factor=d_factor, conductivity=conductivity)


class MeshFractureTemplate(FractureTemplate):
    """
    Attributes:
        active_time_step_index (int): Active TimeStep Index
        conductivity_result_name (str): Active Conductivity Result Name
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.active_time_step_index = 0
        self.conductivity_result_name = ""
        FractureTemplate.__init__(self, pb2_object, channel)
        if MeshFractureTemplate.__custom_init__ is not None:
            MeshFractureTemplate.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class StimPlanFractureTemplate(MeshFractureTemplate):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        MeshFractureTemplate.__init__(self, pb2_object, channel)
        if StimPlanFractureTemplate.__custom_init__ is not None:
            StimPlanFractureTemplate.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class StimPlanModel(CheckableNamedObject):
    """
    Attributes:
        anchor_position (str): Anchor Position
        auto_compute_barrier (bool): Auto Compute Barrier
        azimuth_angle (float): Azimuth Angle
        barrier (bool): Barrier
        barrier_dip (float): Barrier Dip
        barrier_fault_name (str): Barrier Fault
        barrier_text_annotation (str): Barrier Text Annotation
        bounding_box_horizontal (float): Bounding Box Horizontal
        bounding_box_vertical (float): Bounding Box Vertical
        distance_to_barrier (float): Distance To Barrier [m]
        eclipse_case (str): Dynamic Case
        extraction_depth_bottom (float): Depth
        extraction_depth_top (float): Depth
        extraction_offset_bottom (float): Bottom Offset
        extraction_offset_top (float): Top Offset
        extraction_type (str): One of [TVT, TST]
        formation_dip (float): Formation Dip
        fracture_orientation (str): One of [Longitudinal, Transverse, Azimuth]
        initial_pressure_eclipse_case (str): Initial Pressure Case
        measured_depth (float): Measured Depth
        original_thickness_direction (str): Original Thickness Direction
        perforation_interval (str): Perforation Interval
        perforation_length (float): Perforation Length [m]
        poro_elastic_constant (float): Poro-Elastic Constant
        relative_permeability_factor (float): Relative Permeability Factor
        show_all_faults (bool): Show All Faults
        show_only_barrier_fault (bool): Show Only Barrier Fault
        static_eclipse_case (str): Static Case
        thermal_expansion_coefficient (float): Thermal Expansion Coefficient [1/C]
        thickness_direction (str): Thickness Direction
        thickness_direction_well_path (str): Thickness Direction Well Path
        time_step (int): Time Step
        use_detailed_fluid_loss (bool): Use Detailed Fluid Loss
        well_penetration_layer (int): Well Penetration Layer
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.anchor_position = [0, 0, 0]
        self.auto_compute_barrier = True
        self.azimuth_angle = 0
        self.barrier = True
        self.barrier_dip = 0
        self.barrier_fault_name = ""
        self.barrier_text_annotation = ""
        self.bounding_box_horizontal = 50
        self.bounding_box_vertical = 100
        self.distance_to_barrier = 0
        self.eclipse_case = ""
        self.extraction_depth_bottom = -1
        self.extraction_depth_top = -1
        self.extraction_offset_bottom = -1
        self.extraction_offset_top = -1
        self.extraction_type = "TST"
        self.formation_dip = 0
        self.fracture_orientation = "Longitudinal"
        self.initial_pressure_eclipse_case = ""
        self.measured_depth = 0
        self.original_thickness_direction = [0, 0, 0]
        self.perforation_interval = ""
        self.perforation_length = 10
        self.poro_elastic_constant = 0
        self.relative_permeability_factor = 0.5
        self.show_all_faults = False
        self.show_only_barrier_fault = False
        self.static_eclipse_case = ""
        self.thermal_expansion_coefficient = 0
        self.thickness_direction = [0, 0, 0]
        self.thickness_direction_well_path = ""
        self.time_step = 0
        self.use_detailed_fluid_loss = True
        self.well_penetration_layer = 2
        CheckableNamedObject.__init__(self, pb2_object, channel)
        if StimPlanModel.__custom_init__ is not None:
            StimPlanModel.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def export_to_file(self, directory_path=""):
        """
        Export StimPlan Model Plot to File

        Arguments:
            directory_path (str): Directory Path
        Returns:
            StimPlanModel
        """
        return self._call_pdm_method("ExportToFile", directory_path=directory_path)


class StimPlanModelCollection(CheckableNamedObject):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        CheckableNamedObject.__init__(self, pb2_object, channel)
        if StimPlanModelCollection.__custom_init__ is not None:
            StimPlanModelCollection.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def append_stim_plan_model(self, well_path="", measured_depth=0, stim_plan_model_template=""):
        """
        Create a new StimPlan Model

        Arguments:
            well_path (WellPathBase): Well Path
            measured_depth (float): 
            stim_plan_model_template (StimPlanModelTemplate): StimPlan Model Template
        Returns:
            StimPlanModel
        """
        return self._call_pdm_method("AppendStimPlanModel", well_path=well_path, measured_depth=measured_depth, stim_plan_model_template=stim_plan_model_template)


    def stim_plan_models(self):
        """

        Returns:
             List of StimPlanModel
        """
        return self.children("StimPlanModels", StimPlanModel)


class PlotWindow(ViewWindow):
    """
    The Abstract base class for all MDI Windows in the Plot Window

    Attributes:
        id (int): View ID
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.id = -1
        ViewWindow.__init__(self, pb2_object, channel)
        if PlotWindow.__custom_init__ is not None:
            PlotWindow.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class DepthTrackPlot(PlotWindow):
    """
    Attributes:
        auto_scale_depth_enabled (bool): Auto Scale
        auto_zoom_max_depth_factor (float): Auto Zoom Maximum Factor
        auto_zoom_min_depth_factor (float): Auto Zoom Minimum Factor
        axis_title_font_size (str): One of [XX_Small, X_Small, Small, Medium, Large, X_Large, XX_Large]
        axis_value_font_size (str): One of [XX_Small, X_Small, Small, Medium, Large, X_Large, XX_Large]
        depth_type (str): One of [MEASURED_DEPTH, TRUE_VERTICAL_DEPTH, PSEUDO_LENGTH, CONNECTION_NUMBER, TRUE_VERTICAL_DEPTH_RKB]
        depth_unit (str): One of [UNIT_METER, UNIT_FEET, UNIT_NONE]
        maximum_depth (float): Max
        minimum_depth (float): Min
        show_depth_grid_lines (str): One of [GRID_X_NONE, GRID_X_MAJOR, GRID_X_MAJOR_AND_MINOR]
        show_depth_marker_line (bool): Show Depth Marker Line
        sub_title_font_size (str): One of [XX_Small, X_Small, Small, Medium, Large, X_Large, XX_Large]
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.auto_scale_depth_enabled = True
        self.auto_zoom_max_depth_factor = 0
        self.auto_zoom_min_depth_factor = 0
        self.axis_title_font_size = "Medium"
        self.axis_value_font_size = "Medium"
        self.depth_type = "MEASURED_DEPTH"
        self.depth_unit = "UNIT_METER"
        self.maximum_depth = 1000
        self.minimum_depth = 0
        self.show_depth_grid_lines = "GRID_X_MAJOR"
        self.show_depth_marker_line = False
        self.sub_title_font_size = "Medium"
        PlotWindow.__init__(self, pb2_object, channel)
        if DepthTrackPlot.__custom_init__ is not None:
            DepthTrackPlot.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class StimPlanModelPlot(DepthTrackPlot):
    """
    A fracture model plot

    Attributes:
        eclipse_case (str): Case
        stim_plan_model (str): StimPlan Model
        time_step (int): Time Step
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.eclipse_case = ""
        self.stim_plan_model = ""
        self.time_step = 0
        DepthTrackPlot.__init__(self, pb2_object, channel)
        if StimPlanModelPlot.__custom_init__ is not None:
            StimPlanModelPlot.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class StimPlanModelPlotCollection(PdmObjectBase):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        PdmObjectBase.__init__(self, pb2_object, channel)
        if StimPlanModelPlotCollection.__custom_init__ is not None:
            StimPlanModelPlotCollection.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def append_stim_plan_model_plot(self, stim_plan_model=""):
        """
        Create a new StimPlan Model

        Arguments:
            stim_plan_model (StimPlanModel): StimPlan Model
        Returns:
            StimPlanModelPlot
        """
        return self._call_pdm_method("AppendStimPlanModelPlot", stim_plan_model=stim_plan_model)


    def stim_plan_model_plots(self):
        """

        Returns:
             List of StimPlanModelPlot
        """
        return self.children("StimPlanModelPlots", StimPlanModelPlot)


class StimPlanModelTemplate(NamedObject):
    """
    Attributes:
        default_facies (str): Default Facies
        default_permeability (float): Default Permeability
        default_porosity (float): Default Porosity
        dynamic_eclipse_case (str): Dynamic Case
        id (int): ID
        initial_pressure_eclipse_case (str): Initial Pressure Case
        overburden_facies (str): Overburden Facies
        overburden_fluid_density (float): Overburden Fluid Density [g/cm^3]
        overburden_formation (str): Overburden Formation
        overburden_height (float): Overburden Height
        overburden_permeability (float): Overburden Permeability
        overburden_porosity (float): Overburden Porosity
        reference_temperature (float): Temperature [C]
        reference_temperature_depth (float): Temperature Depth [m]
        reference_temperature_gradient (float): Temperature Gradient [C/m]
        static_eclipse_case (str): Static Case
        stress_depth (float): Stress Depth
        time_step (int): Time Step
        underburden_facies (str): Underburden Facies
        underburden_fluid_density (float): Underburden Fluid Density [g/cm^3]
        underburden_formation (str): Underburden Formation
        underburden_height (float): Underburden Height
        underburden_permeability (float): Underburden Permeability
        underburden_porosity (float): Underburden Porosity
        vertical_stress (float): Vertical Stress
        vertical_stress_gradient (float): Vertical Stress Gradient
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.default_facies = ""
        self.default_permeability = 0.0001
        self.default_porosity = 0.01
        self.dynamic_eclipse_case = ""
        self.id = -1
        self.initial_pressure_eclipse_case = ""
        self.overburden_facies = ""
        self.overburden_fluid_density = 1.03
        self.overburden_formation = ""
        self.overburden_height = 50
        self.overburden_permeability = 1e-05
        self.overburden_porosity = 0
        self.reference_temperature = 70
        self.reference_temperature_depth = 2500
        self.reference_temperature_gradient = 0.025
        self.static_eclipse_case = ""
        self.stress_depth = 1000
        self.time_step = 0
        self.underburden_facies = ""
        self.underburden_fluid_density = 1.03
        self.underburden_formation = ""
        self.underburden_height = 50
        self.underburden_permeability = 1e-05
        self.underburden_porosity = 0
        self.vertical_stress = 238
        self.vertical_stress_gradient = 0.238
        NamedObject.__init__(self, pb2_object, channel)
        if StimPlanModelTemplate.__custom_init__ is not None:
            StimPlanModelTemplate.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def elastic_properties(self):
        """Elastic Properties

        Returns:
             ElasticProperties
        """
        children = self.children("ElasticProperties", ElasticProperties)
        return children[0] if len(children) > 0 else None


    def facies_initial_pressure_configs(self):
        """Facies Initial Pressure Configs

        Returns:
             List of FaciesInitialPressureConfig
        """
        return self.children("FaciesInitialPressureConfigs", FaciesInitialPressureConfig)


    def facies_properties(self):
        """Facies Properties

        Returns:
             FaciesProperties
        """
        children = self.children("FaciesProperties", FaciesProperties)
        return children[0] if len(children) > 0 else None


    def non_net_layers(self):
        """Non-Net Layers

        Returns:
             NonNetLayers
        """
        children = self.children("NonNetLayers", NonNetLayers)
        return children[0] if len(children) > 0 else None


    def pressure_table(self):
        """Pressure Table

        Returns:
             PressureTable
        """
        children = self.children("PressureTable", PressureTable)
        return children[0] if len(children) > 0 else None


class StimPlanModelTemplateCollection(PdmObjectBase):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        PdmObjectBase.__init__(self, pb2_object, channel)
        if StimPlanModelTemplateCollection.__custom_init__ is not None:
            StimPlanModelTemplateCollection.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def append_stim_plan_model_template(self, eclipse_case="", time_step=0, facies_properties_file_path="", elastic_properties_file_path=""):
        """
        Create a new StimPlan Model Template

        Arguments:
            eclipse_case (RimReservoir): Eclipse Case
            time_step (int): Time Step
            facies_properties_file_path (str): Facies Properties File Path
            elastic_properties_file_path (str): Elastic Properties File Path
        Returns:
            StimPlanModelTemplate
        """
        return self._call_pdm_method("AppendStimPlanModelTemplate", eclipse_case=eclipse_case, time_step=time_step, facies_properties_file_path=facies_properties_file_path, elastic_properties_file_path=elastic_properties_file_path)


    def stim_plan_model_templates(self):
        """StimPlan Model Templates

        Returns:
             List of StimPlanModelTemplate
        """
        return self.children("StimPlanModelTemplates", StimPlanModelTemplate)


class SummaryCaseSubCollection(PdmObjectBase):
    """
    Attributes:
        create_auto_name (bool): Auto Name
        id (int): Ensemble ID
        is_ensemble (bool): Is Ensemble
        name_count (str): Name
        summary_collection_name (str): Name
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.create_auto_name = True
        self.id = -1
        self.is_ensemble = False
        self.name_count = "Group"
        self.summary_collection_name = "Group"
        PdmObjectBase.__init__(self, pb2_object, channel)
        if SummaryCaseSubCollection.__custom_init__ is not None:
            SummaryCaseSubCollection.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class Plot(PlotWindow):
    """
    The Abstract Base Class for all Plot Objects

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        PlotWindow.__init__(self, pb2_object, channel)
        if Plot.__custom_init__ is not None:
            Plot.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class SummaryPlot(Plot):
    """
    A Summary Plot

    Attributes:
        is_using_auto_name (bool): Auto Title
        normalize_curve_y_values (bool): Normalize all curves
        plot_description (str): Name
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.is_using_auto_name = True
        self.normalize_curve_y_values = False
        self.plot_description = "Summary Plot"
        Plot.__init__(self, pb2_object, channel)
        if SummaryPlot.__custom_init__ is not None:
            SummaryPlot.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class SummaryPlotCollection(PdmObjectBase):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        PdmObjectBase.__init__(self, pb2_object, channel)
        if SummaryPlotCollection.__custom_init__ is not None:
            SummaryPlotCollection.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def new_summary_plot(self, summary_cases=[], ensemble="", address=""):
        """
        Create a new Summary Plot

        Arguments:
            summary_cases (List of SummaryCase): Summary Cases
            ensemble (SummaryCaseSubCollection): Ensemble
            address (str): Formatted address string specifying the plot options
        Returns:
            SummaryPlot
        """
        return self._call_pdm_method("NewSummaryPlot", summary_cases=summary_cases, ensemble=ensemble, address=address)


class Surface(SurfaceInterface):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        SurfaceInterface.__init__(self, pb2_object, channel)
        if Surface.__custom_init__ is not None:
            Surface.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class ThermalFractureTemplate(MeshFractureTemplate):
    """
    Attributes:
        filter_cake_pressure_drop (str): One of [None, Relative, Absolute]
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.filter_cake_pressure_drop = "Relative"
        MeshFractureTemplate.__init__(self, pb2_object, channel)
        if ThermalFractureTemplate.__custom_init__ is not None:
            ThermalFractureTemplate.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def export_to_file(self, file_path="", time_step=0):
        """
        Export Thermal Fracture Template to File

        Arguments:
            file_path (str): File Path
            time_step (int): 
        Returns:
            ThermalFractureTemplate
        """
        return self._call_pdm_method("ExportToFile", file_path=file_path, time_step=time_step)


    def time_steps(self, ):
        """
        Get Thermal Fracture Template Time Steps

        Arguments:
            
        Returns:
            DataContainerString
        """
        return self._call_pdm_method("TimeSteps")


class TriangleGeometry(PdmObjectBase):
    """
    Attributes:
        connections (List of int): Indices to triangle vertices
        display_model_offset (str): Display Model Offset
        fault_mesh_x_coords (List of float): Fault Mesh X coords
        fault_mesh_y_coords (List of float): Fault Mesh Y coords
        fault_mesh_z_coords (List of float): Fault Mesh Z coords
        mesh_x_coords (List of float): Mesh X coords
        mesh_y_coords (List of float): Mesh Y coords
        mesh_z_coords (List of float): Mesh Z coords
        x_coords (List of float): X coords
        y_coords (List of float): Y coords
        z_coords (List of float): Z coords
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.connections = []
        self.display_model_offset = [0, 0, 0]
        self.fault_mesh_x_coords = []
        self.fault_mesh_y_coords = []
        self.fault_mesh_z_coords = []
        self.mesh_x_coords = []
        self.mesh_y_coords = []
        self.mesh_z_coords = []
        self.x_coords = []
        self.y_coords = []
        self.z_coords = []
        PdmObjectBase.__init__(self, pb2_object, channel)
        if TriangleGeometry.__custom_init__ is not None:
            TriangleGeometry.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class WbsParameters(PdmObjectBase):
    """
    Attributes:
        df_source (str): One of [GRID, LAS_FILE, ELEMENT_PROPERTY_TABLE, USER_DEFINED, HYDROSTATIC, DERIVED_FROM_K0FG, PROPORTIONAL_TO_SH, UNDEFINED]
        fg_multiplier (float): SH Multiplier for FG in Shale
        fg_shale_source (str): One of [GRID, LAS_FILE, ELEMENT_PROPERTY_TABLE, USER_DEFINED, HYDROSTATIC, DERIVED_FROM_K0FG, PROPORTIONAL_TO_SH, UNDEFINED]
        k0_fg_source (str): One of [GRID, LAS_FILE, ELEMENT_PROPERTY_TABLE, USER_DEFINED, HYDROSTATIC, DERIVED_FROM_K0FG, PROPORTIONAL_TO_SH, UNDEFINED]
        k0_sh_source (str): One of [GRID, LAS_FILE, ELEMENT_PROPERTY_TABLE, USER_DEFINED, HYDROSTATIC, DERIVED_FROM_K0FG, PROPORTIONAL_TO_SH, UNDEFINED]
        obg0_source (str): One of [GRID, LAS_FILE, ELEMENT_PROPERTY_TABLE, USER_DEFINED, HYDROSTATIC, DERIVED_FROM_K0FG, PROPORTIONAL_TO_SH, UNDEFINED]
        poission_ratio_source (str): One of [GRID, LAS_FILE, ELEMENT_PROPERTY_TABLE, USER_DEFINED, HYDROSTATIC, DERIVED_FROM_K0FG, PROPORTIONAL_TO_SH, UNDEFINED]
        pore_pressure_non_reservoir_source (str): One of [GRID, LAS_FILE, ELEMENT_PROPERTY_TABLE, USER_DEFINED, HYDROSTATIC, DERIVED_FROM_K0FG, PROPORTIONAL_TO_SH, UNDEFINED]
        pore_pressure_reservoir_source (str): One of [GRID, LAS_FILE, ELEMENT_PROPERTY_TABLE, USER_DEFINED, HYDROSTATIC, DERIVED_FROM_K0FG, PROPORTIONAL_TO_SH, UNDEFINED]
        ucs_source (str): One of [GRID, LAS_FILE, ELEMENT_PROPERTY_TABLE, USER_DEFINED, HYDROSTATIC, DERIVED_FROM_K0FG, PROPORTIONAL_TO_SH, UNDEFINED]
        user_df (float): User Defined DF
        user_k0_fg (float): User Defined K0_FG
        user_k0_sh (float): User Defined K0_SH
        user_poisson_ratio (float): User Defined Poisson Ratio
        user_pp_non_reservoir (float):   Multiplier of hydrostatic PP
        user_ucs (float): User Defined UCS [bar]
        water_density (float): Density of Sea Water [g/cm^3]
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.df_source = "LAS_FILE"
        self.fg_multiplier = 1.05
        self.fg_shale_source = "DERIVED_FROM_K0FG"
        self.k0_fg_source = "LAS_FILE"
        self.k0_sh_source = "LAS_FILE"
        self.obg0_source = "GRID"
        self.poission_ratio_source = "LAS_FILE"
        self.pore_pressure_non_reservoir_source = "LAS_FILE"
        self.pore_pressure_reservoir_source = "GRID"
        self.ucs_source = "LAS_FILE"
        self.user_df = 0.7
        self.user_k0_fg = 0.75
        self.user_k0_sh = 0.65
        self.user_poisson_ratio = 0.35
        self.user_pp_non_reservoir = 1
        self.user_ucs = 100
        self.water_density = 1.03
        PdmObjectBase.__init__(self, pb2_object, channel)
        if WbsParameters.__custom_init__ is not None:
            WbsParameters.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class SimulationWell(PdmObjectBase):
    """
    An Eclipse Simulation Well

    Attributes:
        name (str): Name
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.name = ""
        PdmObjectBase.__init__(self, pb2_object, channel)
        if SimulationWell.__custom_init__ is not None:
            SimulationWell.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class WellLogPlot(DepthTrackPlot):
    """
    A Well Log Plot With a shared Depth Axis and Multiple Tracks

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        DepthTrackPlot.__init__(self, pb2_object, channel)
        if WellLogPlot.__custom_init__ is not None:
            WellLogPlot.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def new_well_log_track(self, title="", case="", well_path=""):
        """
        Create a new well log track

        Arguments:
            title (str): Title
            case (RimReservoir): Case
            well_path (WellPathBase): Well Path
        Returns:
            WellLogPlotTrack
        """
        return self._call_pdm_method("NewWellLogTrack", title=title, case=case, well_path=well_path)


class WellBoreStabilityPlot(WellLogPlot):
    """
    A GeoMechanical Well Bore Stability Plot

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        WellLogPlot.__init__(self, pb2_object, channel)
        if WellBoreStabilityPlot.__custom_init__ is not None:
            WellBoreStabilityPlot.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def parameters(self):
        """Well Bore Stability Parameters

        Returns:
             WbsParameters
        """
        children = self.children("Parameters", WbsParameters)
        return children[0] if len(children) > 0 else None


class PlotCurve(PdmObjectBase):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        PdmObjectBase.__init__(self, pb2_object, channel)
        if PlotCurve.__custom_init__ is not None:
            PlotCurve.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class WellLogPlotCurve(PlotCurve):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        PlotCurve.__init__(self, pb2_object, channel)
        if WellLogPlotCurve.__custom_init__ is not None:
            WellLogPlotCurve.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class WellLogExtractionCurve(WellLogPlotCurve):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        WellLogPlotCurve.__init__(self, pb2_object, channel)
        if WellLogExtractionCurve.__custom_init__ is not None:
            WellLogExtractionCurve.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class WellLogPlotCollection(PdmObjectBase):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        PdmObjectBase.__init__(self, pb2_object, channel)
        if WellLogPlotCollection.__custom_init__ is not None:
            WellLogPlotCollection.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def new_well_log_plot(self, case="", well_path="", property_type="", property_name="", time_step=0):
        """
        Create a new well log plot

        Arguments:
            case (RimReservoir): Case
            well_path (WellPathBase): Well Path
            property_type (str): Property Type
            property_name (str): Property Name
            time_step (int): Time Step
        Returns:
            WellLogPlot
        """
        return self._call_pdm_method("NewWellLogPlot", case=case, well_path=well_path, property_type=property_type, property_name=property_name, time_step=time_step)


    def well_log_plots(self):
        """

        Returns:
             List of WellLogPlot
        """
        return self.children("WellLogPlots", WellLogPlot)


class WellLogPlotTrack(Plot):
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        Plot.__init__(self, pb2_object, channel)
        if WellLogPlotTrack.__custom_init__ is not None:
            WellLogPlotTrack.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def add_extraction_curve(self, case="", well_path="", property_type="", property_name="", time_step=0):
        """
        Create a well log extraction curve

        Arguments:
            case (RimReservoir): Case
            well_path (WellPathBase): Well Path
            property_type (str): Property Type
            property_name (str): Property Name
            time_step (int): Time Step
        Returns:
            WellLogExtractionCurve
        """
        return self._call_pdm_method("AddExtractionCurve", case=case, well_path=well_path, property_type=property_type, property_name=property_name, time_step=time_step)


class FileWellPath(WellPath):
    """
    Well Paths Loaded From File

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        WellPath.__init__(self, pb2_object, channel)
        if FileWellPath.__custom_init__ is not None:
            FileWellPath.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class WellPathGeometry(PdmObjectBase):
    """
    Class containing the geometry of a modeled Well Path

    Attributes:
        air_gap (float): Air Gap
        attached_to_parent_well (bool): Attached to Parent Well
        link_reference_point_updates (bool): Link Reference Point
        md_at_first_target (float): MD at First Target
        reference_point (str): UTM Reference Point
        show_spheres (bool): Spheres
        use_auto_generated_target_at_sea_level (bool): Generate Target at Sea Level
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.air_gap = 0
        self.attached_to_parent_well = False
        self.link_reference_point_updates = False
        self.md_at_first_target = 0
        self.reference_point = [0, 0, 0]
        self.show_spheres = True
        self.use_auto_generated_target_at_sea_level = True
        PdmObjectBase.__init__(self, pb2_object, channel)
        if WellPathGeometry.__custom_init__ is not None:
            WellPathGeometry.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def append_well_target(self, coordinate=[0, 0, 0], absolute=False, use_fixed_azimuth=False, use_fixed_inclination=False, fixed_azimuth_value=0, fixed_inclination_value=0):
        """
        Create and Add New Well Target

        Arguments:
            coordinate (N3cvf7Vector3IdEE): Coordinate
            absolute (bool): Relative or Absolute Coordinate
            use_fixed_azimuth (bool): 
            use_fixed_inclination (bool): 
            fixed_azimuth_value (float): [Degrees]
            fixed_inclination_value (float): [Degrees]
        Returns:
            WellPathTarget
        """
        return self._call_pdm_method("AppendWellTarget", coordinate=coordinate, absolute=absolute, use_fixed_azimuth=use_fixed_azimuth, use_fixed_inclination=use_fixed_inclination, fixed_azimuth_value=fixed_azimuth_value, fixed_inclination_value=fixed_inclination_value)


    def auto_generated_target(self):
        """Auto Generated Target

        Returns:
             WellPathTarget
        """
        children = self.children("AutoGeneratedTarget", WellPathTarget)
        return children[0] if len(children) > 0 else None


    def well_path_targets(self):
        """Well Targets

        Returns:
             List of WellPathTarget
        """
        return self.children("WellPathTargets", WellPathTarget)


class WellPathTarget(PdmObjectBase):
    """
    Class containing the Well Target definition

    Attributes:
        azimuth (float): Azi(deg)
        dogleg1 (float): DL in
        dogleg2 (float): DL out
        estimated_azimuth (float): Est Azi(deg)
        estimated_dogleg1 (float): Est DL in
        estimated_dogleg2 (float): Est DL out
        estimated_inclination (float): Est Inc(deg)
        inclination (float): Inc(deg)
        target_measured_depth (float): MD
        target_point (str): Relative Coord
        use_fixed_azimuth (bool): Azi
        use_fixed_inclination (bool): Inc
    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        self.azimuth = 0
        self.dogleg1 = 3
        self.dogleg2 = 3
        self.estimated_azimuth = 0
        self.estimated_dogleg1 = 0
        self.estimated_dogleg2 = 0
        self.estimated_inclination = 0
        self.inclination = 0
        self.target_measured_depth = 0
        self.target_point = [0, 0, 0]
        self.use_fixed_azimuth = False
        self.use_fixed_inclination = False
        PdmObjectBase.__init__(self, pb2_object, channel)
        if WellPathTarget.__custom_init__ is not None:
            WellPathTarget.__custom_init__(self, pb2_object=pb2_object, channel=channel)

class WellPathCollection(PdmObjectBase):
    """
    Collection of Well Paths

    """
    __custom_init__ = None #: Assign a custom init routine to be run at __init__

    def __init__(self, pb2_object=None, channel=None):
        PdmObjectBase.__init__(self, pb2_object, channel)
        if WellPathCollection.__custom_init__ is not None:
            WellPathCollection.__custom_init__(self, pb2_object=pb2_object, channel=channel)

    def well_paths(self):
        """Well Paths

        Returns:
             List of WellPath
        """
        return self.children("WellPaths", WellPath)


def class_dict():
    classes = {}
    classes['Case'] = Case
    classes['CellColors'] = CellColors
    classes['CellFilterCollection'] = CellFilterCollection
    classes['CheckableNamedObject'] = CheckableNamedObject
    classes['CommandRouter'] = CommandRouter
    classes['CurveIntersection'] = CurveIntersection
    classes['DataContainerFloat'] = DataContainerFloat
    classes['DataContainerString'] = DataContainerString
    classes['DataContainerTime'] = DataContainerTime
    classes['DepthTrackPlot'] = DepthTrackPlot
    classes['EclipseCase'] = EclipseCase
    classes['EclipseContourMap'] = EclipseContourMap
    classes['EclipseResult'] = EclipseResult
    classes['EclipseView'] = EclipseView
    classes['ElasticProperties'] = ElasticProperties
    classes['ElasticPropertyScaling'] = ElasticPropertyScaling
    classes['ElasticPropertyScalingCollection'] = ElasticPropertyScalingCollection
    classes['EnsembleStatisticsSurface'] = EnsembleStatisticsSurface
    classes['EnsembleSurface'] = EnsembleSurface
    classes['EnsembleWellLogs'] = EnsembleWellLogs
    classes['FaciesInitialPressureConfig'] = FaciesInitialPressureConfig
    classes['FaciesProperties'] = FaciesProperties
    classes['FileSummaryCase'] = FileSummaryCase
    classes['FileWellPath'] = FileWellPath
    classes['FractureTemplate'] = FractureTemplate
    classes['FractureTemplateCollection'] = FractureTemplateCollection
    classes['GeoMechCase'] = GeoMechCase
    classes['GeoMechContourMap'] = GeoMechContourMap
    classes['GeoMechPart'] = GeoMechPart
    classes['GeoMechPartCollection'] = GeoMechPartCollection
    classes['GeoMechView'] = GeoMechView
    classes['GridCaseGroup'] = GridCaseGroup
    classes['GridCaseSurface'] = GridCaseSurface
    classes['GridSummaryCase'] = GridSummaryCase
    classes['IntersectionCollection'] = IntersectionCollection
    classes['MeshFractureTemplate'] = MeshFractureTemplate
    classes['ModeledWellPath'] = ModeledWellPath
    classes['MudWeightWindowParameters'] = MudWeightWindowParameters
    classes['NamedObject'] = NamedObject
    classes['NonNetLayers'] = NonNetLayers
    classes['PdmObjectBase'] = PdmObjectBase
    classes['Plot'] = Plot
    classes['PlotCurve'] = PlotCurve
    classes['PlotWindow'] = PlotWindow
    classes['PressureTable'] = PressureTable
    classes['PressureTableItem'] = PressureTableItem
    classes['Project'] = Project
    classes['ResampleData'] = ResampleData
    classes['Reservoir'] = Reservoir
    classes['SimulationWell'] = SimulationWell
    classes['StimPlanFractureTemplate'] = StimPlanFractureTemplate
    classes['StimPlanModel'] = StimPlanModel
    classes['StimPlanModelCollection'] = StimPlanModelCollection
    classes['StimPlanModelPlot'] = StimPlanModelPlot
    classes['StimPlanModelPlotCollection'] = StimPlanModelPlotCollection
    classes['StimPlanModelTemplate'] = StimPlanModelTemplate
    classes['StimPlanModelTemplateCollection'] = StimPlanModelTemplateCollection
    classes['SummaryCase'] = SummaryCase
    classes['SummaryCaseSubCollection'] = SummaryCaseSubCollection
    classes['SummaryPlot'] = SummaryPlot
    classes['SummaryPlotCollection'] = SummaryPlotCollection
    classes['Surface'] = Surface
    classes['SurfaceCollection'] = SurfaceCollection
    classes['SurfaceInterface'] = SurfaceInterface
    classes['ThermalFractureTemplate'] = ThermalFractureTemplate
    classes['TriangleGeometry'] = TriangleGeometry
    classes['View'] = View
    classes['ViewWindow'] = ViewWindow
    classes['WbsParameters'] = WbsParameters
    classes['WellBoreStabilityPlot'] = WellBoreStabilityPlot
    classes['WellLogExtractionCurve'] = WellLogExtractionCurve
    classes['WellLogPlot'] = WellLogPlot
    classes['WellLogPlotCollection'] = WellLogPlotCollection
    classes['WellLogPlotCurve'] = WellLogPlotCurve
    classes['WellLogPlotTrack'] = WellLogPlotTrack
    classes['WellPath'] = WellPath
    classes['WellPathCollection'] = WellPathCollection
    classes['WellPathGeometry'] = WellPathGeometry
    classes['WellPathTarget'] = WellPathTarget
    return classes

def class_from_keyword(class_keyword):
    all_classes = class_dict()
    if class_keyword in all_classes.keys():
        return all_classes[class_keyword]
    return None



##############################################################################
# This example will create a derived result for each time step synchronously
##############################################################################

import rips
import time

resinsight = rips.Instance.find()
start = time.time()
case = resinsight.project.cases()[0]

# Read the full porv result
porv_results = case.active_cell_property("STATIC_NATIVE", "PORV", 0)
time_step_info = case.time_steps()

for i in range(0, len(time_step_info)):
    # Read the full SOIl result for time step i
    soil_results = case.active_cell_property("DYNAMIC_NATIVE", "SOIL", i)

    # Generate the result by looping through both lists in order
    results = []
    for soil, porv in zip(soil_results, porv_results):
        results.append(soil * porv)

    # Send back result
    case.set_active_cell_property(results, "GENERATED", "SOILPORVSync", i)

end = time.time()
print("Time elapsed: ", end - start)

print("Transferred all results back")

view = case.views()[0].apply_cell_result("GENERATED", "SOILPORVSync")



import sys
import os
import grpc
import pytest

import rips.generated.NNCProperties_pb2 as NNCProperties_pb2

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips

import dataroot


def test_10kSync(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=casePath)

    properties = case.available_nnc_properties()
    assert len(properties) == 1

    assert "TRAN" == properties[0].name
    assert (
        NNCProperties_pb2.NNCPropertyType.Value("NNC_STATIC")
        == properties[0].property_type
    )

    nnc_connections = case.nnc_connections()
    assert len(nnc_connections) == 3416

    connection = nnc_connections[0]
    assert connection.cell1.i == 33
    assert connection.cell1.j == 40
    assert connection.cell1.k == 14
    assert connection.cell_grid_index1 == 0

    tran_vals = case.nnc_connections_static_values("TRAN")
    assert len(tran_vals) == len(nnc_connections)

    for t in tran_vals:
        assert isinstance(t, float)

    # Generate some data
    new_data = []
    for c, _ in enumerate(nnc_connections):
        new_data.append(float(c))

    property_name = "NEW_PROP"
    case.set_nnc_connections_values(new_data, property_name, 0)
    new_prop_vals = case.nnc_connections_generated_values(property_name, 0)
    assert len(new_prop_vals) == len(new_data)
    for i in range(0, len(new_data)):
        assert new_data[i] == new_prop_vals[i]

    # Set some other data for second time step
    for i in range(0, len(new_data)):
        new_data[i] = new_data[i] * 2.0

    case.set_nnc_connections_values(new_data, property_name, 1)
    new_prop_vals = case.nnc_connections_generated_values(property_name, 1)
    assert len(new_prop_vals) == len(nnc_connections)
    for i in range(0, len(new_data)):
        assert new_data[i] == new_prop_vals[i]


def test_non_existing_dynamic_values(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=casePath)

    with pytest.raises(grpc.RpcError):
        case.nnc_connections_dynamic_values("x", 0)


def test_invalid_time_steps(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=casePath)
    with pytest.raises(grpc.RpcError):
        case.nnc_connections_generated_values("Formation Allan", 9999)



# pylint: disable=no-member
# pylint: disable=too-many-arguments
# pylint: disable=too-many-public-methods
# pylint: disable=no-self-use

"""
Module containing the Case class

ResInsight Case class

Operate on a ResInsight case specified by a Case Id integer.
Not meant to be constructed separately but created by one of the following
methods in Project: loadCase, case, allCases, selectedCases

.. _result-definition-label:

Result Definition
-----------------
When working with grid case results, the following two arguments are used in many functions to identify a
result

**Result Definition enums**::
    
    property_type           |       | porosity_model
    (str enum)              |       | (str enum)
    ----------------------- | ----- | --------------
    DYNAMIC_NATIVE          |       | MATRIX_MODEL
    STATIC_NATIVE           |       | FRACTURE_MODEL
    SOURSIMRL               |       |
    GENERATED               |       |
    INPUT_PROPERTY          |       |
    FORMATION_NAMES         |       |
    FLOW_DIAGNOSTICS        |       |
    INJECTION_FLOODING      |       |

"""

import builtins
import grpc

import Case_pb2
import Case_pb2_grpc
import Commands_pb2 as Cmd
import PdmObject_pb2 as PdmObject_pb2

import Properties_pb2
import Properties_pb2_grpc
import NNCProperties_pb2
import NNCProperties_pb2_grpc
from .resinsight_classes import (
    Case,
    EclipseCase,
    GeoMechCase,
    WellBoreStabilityPlot,
    WbsParameters,
)

from .grid import Grid
from .pdmobject import add_method
from .view import View
from .simulation_well import SimulationWell
import rips.project  # full name import due to circular dependency


@add_method(Case)
def __custom_init__(self, pb2_object, channel):
    self.__case_stub = Case_pb2_grpc.CaseStub(self._channel)

    self.__properties_stub = Properties_pb2_grpc.PropertiesStub(self._channel)
    self.__nnc_properties_stub = NNCProperties_pb2_grpc.NNCPropertiesStub(self._channel)

    # Public properties
    self.chunk_size = 8160


@add_method(Case)
def __grid_count(self):
    """Get number of grids in the case"""
    try:
        return self.__case_stub.GetGridCount(self.__request()).count
    except grpc.RpcError as exception:
        if exception.code() == grpc.StatusCode.NOT_FOUND:
            return 0
        return 0


@add_method(Case)
def __request(self):
    return Case_pb2.CaseRequest(id=self.id)


@add_method(Case)
def __generate_property_input_iterator(self, values_iterator, parameters):
    chunk = Properties_pb2.PropertyInputChunk()
    chunk.params.CopyFrom(parameters)
    yield chunk

    for values in values_iterator:
        valmsg = Properties_pb2.PropertyChunk(values=values)
        chunk.values.CopyFrom(valmsg)
        yield chunk


@add_method(Case)
def __generate_property_input_chunks(self, array, parameters):
    index = -1
    while index < len(array):
        chunk = Properties_pb2.PropertyInputChunk()
        if index == -1:
            chunk.params.CopyFrom(parameters)
            index += 1
        else:
            actual_chunk_size = min(len(array) - index + 1, self.chunk_size)
            chunk.values.CopyFrom(
                Properties_pb2.PropertyChunk(
                    values=array[index : index + actual_chunk_size]
                )
            )
            index += actual_chunk_size

        yield chunk
    # Final empty message to signal completion
    chunk = Properties_pb2.PropertyInputChunk()
    yield chunk


@add_method(Case)
def grid(self, index=0):
    """Get Grid of a given index

    Arguments:
        index (int): The grid index

    Returns:
        :class:`rips.grid.Grid`
    """
    return Grid(index, self, self.channel())


@add_method(Case)
def grids(self):
    """Get a list of all rips Grid objects in the case

    Returns:
        List of :class:`rips.grid.Grid`
    """
    grid_list = []
    for i in range(0, self.__grid_count()):
        grid_list.append(Grid(i, self, self.channel()))
    return grid_list


@add_method(Case)
def replace(self, new_grid_file):
    """Replace the current case grid with a new grid loaded from file

    Arguments:
        new_egrid_file (str): Path to EGRID file
    """
    project = self.ancestor(rips.project.Project)
    self._execute_command(
        replaceCase=Cmd.ReplaceCaseRequest(newGridFile=new_grid_file, caseId=self.id)
    )
    new_case = project.case(self.id)
    self.copy_from(new_case)


@add_method(Case)
def cell_count(self, porosity_model="MATRIX_MODEL"):
    """Get a cell count object containing number of active cells and total number of cells

    Arguments:
        porosity_model (str): String representing an enum.
            must be 'MATRIX_MODEL' or 'FRACTURE_MODEL'.
    Returns:
        Cell Count object with the following integer attributes:
            active_cell_count: number of active cells
            reservoir_cell_count: total number of reservoir cells

    **CellCount class description**::

        Parameter               | Description               | Type
        ----------------------- | ------------------------- | -----
        active_cell_count       | Number of active cells    | Integer
        reservoir_cell_count    | Total number of cells     | Integer

    """
    porosity_model_enum = Case_pb2.PorosityModelType.Value(porosity_model)
    request = Case_pb2.CellInfoRequest(
        case_request=self.__request(), porosity_model=porosity_model_enum
    )
    return self.__case_stub.GetCellCount(request)


@add_method(Case)
def cell_info_for_active_cells_async(self, porosity_model="MATRIX_MODEL"):
    """Get Stream of cell info objects for current case

    Arguments:
        porosity_model(str): String representing an enum.
            must be 'MATRIX_MODEL' or 'FRACTURE_MODEL'.

    Returns:
        Stream of **CellInfo** objects

    See :meth:`rips.case.cell_info_for_active_cells()` for detalis on the **CellInfo** class.
    """
    porosity_model_enum = Case_pb2.PorosityModelType.Value(porosity_model)
    request = Case_pb2.CellInfoRequest(
        case_request=self.__request(), porosity_model=porosity_model_enum
    )
    return self.__case_stub.GetCellInfoForActiveCells(request)


@add_method(Case)
def cell_info_for_active_cells(self, porosity_model="MATRIX_MODEL"):
    """Get list of cell info objects for current case

    Arguments:
        porosity_model(str): String representing an enum.
            must be 'MATRIX_MODEL' or 'FRACTURE_MODEL'.

    Returns:
        List of **CellInfo** objects

    **CellInfo class description**::

        Parameter                 | Description                                   | Type
        ------------------------- | --------------------------------------------- | -----
        grid_index                | Index to grid                                 | Integer
        parent_grid_index         | Index to parent grid                          | Integer
        coarsening_box_index      | Index to coarsening box                       | Integer
        local_ijk                 | Cell index in IJK directions of local grid    | Vec3i
        parent_ijk                | Cell index in IJK directions of parent grid   | Vec3i

    **Vec3i class description**::

        Parameter        | Description                                  | Type
        ---------------- | -------------------------------------------- | -----
        i                | I grid index                                 | Integer
        j                | J grid index                                 | Integer
        k                | K grid index                                 | Integer

    """
    active_cell_info_chunks = self.cell_info_for_active_cells_async(
        porosity_model=porosity_model
    )
    received_active_cells = []
    for active_cell_chunk in active_cell_info_chunks:
        for active_cell in active_cell_chunk.data:
            received_active_cells.append(active_cell)
    return received_active_cells


@add_method(Case)
def time_steps(self):
    """Get a list containing all time steps

    The time steps are defined by the class **TimeStepDate**

    **TimeStepDate class description**::

        Type      | Name
        --------- | ----------
        int       | year
        int       | month
        int       | day
        int       | hour
        int       | minute
        int       | second


    """
    return self.__case_stub.GetTimeSteps(self.__request()).dates


@add_method(Case)
def reservoir_boundingbox(self):
    """Get the reservoir bounding box

    Returns:
        BoundingBox

    **BoundingBox class description**::

        Type      | Name
        --------- | ----------
        int       | min_x
        int       | max_x
        int       | min_y
        int       | max_y
        int       | min_z
        int       | max_z


    """
    return self.__case_stub.GetReservoirBoundingBox(self.__request())


@add_method(Case)
def reservoir_depth_range(self):
    """Get the reservoir depth range

    Returns:
        A tuple with two members. The first is the minimum depth, the second is the maximum depth
    """
    bbox = self.reservoir_boundingbox()
    return -bbox.max_z, -bbox.min_z


@add_method(Case)
def days_since_start(self):
    """Get a list of decimal values representing days since the start of the simulation"""
    return self.__case_stub.GetDaysSinceStart(self.__request()).day_decimals


@add_method(Case)
def view(self, view_id):
    """Get a particular view belonging to a case by providing view id

    Arguments:
        view_id(int): view id

    Returns:
        :class:`rips.generated.generated_classes.View`
    """
    views = self.views()
    for view_object in views:
        if view_object.id == view_id:
            return view_object
    return None


@add_method(Case)
def create_view(self):
    """Create a new view in the current case

    Returns:
        :class:`rips.generated.generated_classes.View`
    """
    return self.view(
        self._execute_command(
            createView=Cmd.CreateViewRequest(caseId=self.id)
        ).createViewResult.viewId
    )


@add_method(Case)
def export_snapshots_of_all_views(self, prefix="", export_folder=""):
    """Export snapshots for all views in the case

    Arguments:
        prefix (str): Exported file name prefix
        export_folder(str): The path to export to. By default will use the global export folder

    """
    return self._execute_command(
        exportSnapshots=Cmd.ExportSnapshotsRequest(
            type="VIEWS",
            prefix=prefix,
            caseId=self.id,
            viewId=-1,
            exportFolder=export_folder,
        )
    )


@add_method(Case)
def export_well_path_completions(
    self,
    time_step,
    well_path_names,
    file_split,
    compdat_export="TRANSMISSIBILITIES",
    include_perforations=True,
    include_fishbones=True,
    fishbones_exclude_main_bore=True,
    combination_mode="INDIVIDUALLY",
    export_welspec=True,
    export_comments=True,
    custom_file_name="",
):
    """
    Export well path completions for the current case to file

    **Parameters**::

        Parameter                   | Description                                      | Type
        ----------------------------| ------------------------------------------------ | -----
        time_step                   | Time step to export for                          | Integer
        well_path_names             | List of well path names                          | List
        file_split                  | Controls how export data is split into files     | String enum
        compdat_export              | Compdat export type                              | String enum
        include_perforations        | Export perforations?                             | bool
        include_fishbones           | Export fishbones?                                | bool
        fishbones_exclude_main_bore | Exclude main bore when exporting fishbones?      | bool
        combination_mode            | Settings for multiple completions in same cell   | String Enum
        export_welspec              | Export WELSPEC keyword                           | bool
        export_comments             | Export completion data source as comment         | bool
        custom_file_name            | Custom filename when file_split is "UNIFIED_FILE"| String

    **Enum file_split**::

        Option                              | Description
        ----------------------------------- | ------------
        "UNIFIED_FILE"                      | A single file with all combined transmissibilities
        "SPLIT_ON_WELL"                     | One file for each well with combined transmissibilities
        "SPLIT_ON_WELL_AND_COMPLETION_TYPE" | One file for each completion type for each well

    **Enum compdat_export**::

        Option                                      | Description
        ------------------------------------------- | ------------
        "TRANSMISSIBILITIES"                        | Direct export of transmissibilities
        "WPIMULT_AND_DEFAULT_CONNECTION_FACTORS"    | Include WPIMULT in addition to transmissibilities

    **Enum combination_mode**::

        Option              | Description
        ------------------- | ------------
        "INDIVIDUALLY"      | Exports the different completion types into separate sections
        "COMBINED"          | Export one combined transmissibility for each cell

    """
    if isinstance(well_path_names, str):
        well_path_names = [well_path_names]
    return self._execute_command(
        exportWellPathCompletions=Cmd.ExportWellPathCompRequest(
            caseId=self.id,
            timeStep=time_step,
            wellPathNames=well_path_names,
            fileSplit=file_split,
            compdatExport=compdat_export,
            includePerforations=include_perforations,
            includeFishbones=include_fishbones,
            excludeMainBoreForFishbones=fishbones_exclude_main_bore,
            combinationMode=combination_mode,
            exportWelspec=export_welspec,
            exportComments=export_comments,
            customFileName=custom_file_name,
        )
    )


@add_method(Case)
def export_msw(self, well_path):
    """
    Export Eclipse Multi-segment-well model to file

    Arguments:
        well_path(str): Well path name
    """
    return self._execute_command(
        exportMsw=Cmd.ExportMswRequest(caseId=self.id, wellPath=well_path)
    )


@add_method(Case)
def create_multiple_fractures(
    self,
    template_id,
    well_path_names,
    min_dist_from_well_td,
    max_fractures_per_well,
    top_layer,
    base_layer,
    spacing,
    action,
):
    """
    Create Multiple Fractures in one go

    **Parameters**::

        Parameter              | Description                               | Type
        -----------------------| ----------------------------------------- | -----
        template_id            | Id of the template                        | Integer
        well_path_names        | List of well path names                   | List of Strings
        min_dist_from_well_td  | Minimum distance from well TD             | Double
        max_fractures_per_well | Max number of fractures per well          | Integer
        top_layer              | Top grid k-level for fractures            | Integer
        base_layer             | Base grid k-level for fractures           | Integer
        spacing                | Spacing between fractures                 | Double
        action                 | 'APPEND_FRACTURES' or 'REPLACE_FRACTURES' | String enum

    """
    if isinstance(well_path_names, str):
        well_path_names = [well_path_names]
    return self._execute_command(
        createMultipleFractures=Cmd.MultipleFracRequest(
            caseId=self.id,
            templateId=template_id,
            wellPathNames=well_path_names,
            minDistFromWellTd=min_dist_from_well_td,
            maxFracturesPerWell=max_fractures_per_well,
            topLayer=top_layer,
            baseLayer=base_layer,
            spacing=spacing,
            action=action,
        )
    )


@add_method(Case)
def create_lgr_for_completion(
    self,
    time_step,
    well_path_names,
    refinement_i,
    refinement_j,
    refinement_k,
    split_type,
):
    """
    Create a local grid refinement for the completions on the given well paths

    **Parameters**::

        Parameter       | Description                            | Type
        --------------- | -------------------------------------- | -----
        time_steps      | Time step index                        | Integer
        well_path_names | List of well path names                | List of Strings
        refinement_i    | Refinment in x-direction               | Integer
        refinement_j    | Refinment in y-direction               | Integer
        refinement_k    | Refinment in z-direction               | Integer
        split_type      | Defines how to split LGRS              | String enum

    **Enum split_type**::

        Option                  | Description
        ------------------------| ------------
        "LGR_PER_CELL"          | One LGR for each completed cell
        "LGR_PER_COMPLETION"    | One LGR for each completion (fracture, perforation, ...)
        "LGR_PER_WELL"          | One LGR for each well

    """
    if isinstance(well_path_names, str):
        well_path_names = [well_path_names]
    return self._execute_command(
        createLgrForCompletions=Cmd.CreateLgrForCompRequest(
            caseId=self.id,
            timeStep=time_step,
            wellPathNames=well_path_names,
            refinementI=refinement_i,
            refinementJ=refinement_j,
            refinementK=refinement_k,
            splitType=split_type,
        )
    )


@add_method(Case)
def create_saturation_pressure_plots(self):
    """
    Create saturation pressure plots for the current case
    """
    case_ids = [self.id]
    return self._execute_command(
        createSaturationPressurePlots=Cmd.CreateSatPressPlotRequest(caseIds=case_ids)
    )


@add_method(Case)
def export_flow_characteristics(
    self,
    time_steps,
    injectors,
    producers,
    file_name,
    minimum_communication=0.0,
    aquifer_cell_threshold=0.1,
):
    """Export Flow Characteristics data to text file in CSV format

    **Parameters**::

        Parameter                 | Description                                   | Type
        ------------------------- | --------------------------------------------- | -----
        time_steps                | Time step indices                             | List of Integer
        injectors                 | Injector names                                | List of Strings
        producers                 | Producer names                                | List of Strings
        file_name                 | Export file name                              | Integer
        minimum_communication     | Minimum Communication, defaults to 0.0        | Integer
        aquifer_cell_threshold    | Aquifer Cell Threshold, defaults to 0.1       | Integer

    """
    if isinstance(time_steps, int):
        time_steps = [time_steps]
    if isinstance(injectors, str):
        injectors = [injectors]
    if isinstance(producers, str):
        producers = [producers]
    return self._execute_command(
        exportFlowCharacteristics=Cmd.ExportFlowInfoRequest(
            caseId=self.id,
            timeSteps=time_steps,
            injectors=injectors,
            producers=producers,
            fileName=file_name,
            minimumCommunication=minimum_communication,
            aquiferCellThreshold=aquifer_cell_threshold,
        )
    )


@add_method(Case)
def available_properties(self, property_type, porosity_model="MATRIX_MODEL"):
    """Get a list of available properties

    For argument details, see :ref:`Result Definition <result-definition-label>`

    Arguments:
        property_type (str): string corresponding to property_type enum.
        porosity_model(str): 'MATRIX_MODEL' or 'FRACTURE_MODEL'.
    """

    property_type_enum = Properties_pb2.PropertyType.Value(property_type)
    porosity_model_enum = Case_pb2.PorosityModelType.Value(porosity_model)
    request = Properties_pb2.AvailablePropertiesRequest(
        case_request=self.__request(),
        property_type=property_type_enum,
        porosity_model=porosity_model_enum,
    )
    return self.__properties_stub.GetAvailableProperties(request).property_names


@add_method(Case)
def active_cell_property_async(
    self, property_type, property_name, time_step, porosity_model="MATRIX_MODEL"
):
    """Get a cell property for all active cells. Async, so returns an iterator. For argument details, see :ref:`Result Definition <result-definition-label>`

    Arguments:
        property_type(str): string enum
        property_name(str): name of an Eclipse property
        time_step(int): the time step for which to get the property for
        porosity_model(str): string enum

    Returns:
        An iterator to a chunk object containing an array of double values
        Loop through the chunks and then the values within the chunk to get all values.
    """
    property_type_enum = Properties_pb2.PropertyType.Value(property_type)
    porosity_model_enum = Case_pb2.PorosityModelType.Value(porosity_model)
    request = Properties_pb2.PropertyRequest(
        case_request=self.__request(),
        property_type=property_type_enum,
        property_name=property_name,
        time_step=time_step,
        porosity_model=porosity_model_enum,
    )
    for chunk in self.__properties_stub.GetActiveCellProperty(request):
        yield chunk


@add_method(Case)
def active_cell_property(
    self, property_type, property_name, time_step, porosity_model="MATRIX_MODEL"
):
    """Get a cell property for all active cells. Sync, so returns a list. For argument details, see :ref:`Result Definition <result-definition-label>`

    Arguments:
        property_type(str): string enum
        property_name(str): name of an Eclipse property
        time_step(int): the time step for which to get the property for
        porosity_model(str): string enum

    Returns:
        A list containing double values
        Loop through the chunks and then the values within the chunk to get all values.
    """
    all_values = []
    generator = self.active_cell_property_async(
        property_type, property_name, time_step, porosity_model
    )
    for chunk in generator:
        for value in chunk.values:
            all_values.append(value)
    return all_values


@add_method(Case)
def selected_cell_property_async(
    self, property_type, property_name, time_step, porosity_model="MATRIX_MODEL"
):
    """Get a cell property for all selected cells. Async, so returns an iterator. For argument details, see :ref:`Result Definition <result-definition-label>`

    Arguments:
        property_type(str): string enum
        property_name(str): name of an Eclipse property
        time_step(int): the time step for which to get the property for
        porosity_model(str): string enum

    Returns:
        An iterator to a chunk object containing an array of double values
        Loop through the chunks and then the values within the chunk to get all values.
    """
    property_type_enum = Properties_pb2.PropertyType.Value(property_type)
    porosity_model_enum = Case_pb2.PorosityModelType.Value(porosity_model)
    request = Properties_pb2.PropertyRequest(
        case_request=self.__request(),
        property_type=property_type_enum,
        property_name=property_name,
        time_step=time_step,
        porosity_model=porosity_model_enum,
    )
    for chunk in self.__properties_stub.GetSelectedCellProperty(request):
        yield chunk


@add_method(Case)
def selected_cell_property(
    self, property_type, property_name, time_step, porosity_model="MATRIX_MODEL"
):
    """Get a cell property for all selected cells. Sync, so returns a list. For argument details, see :ref:`Result Definition <result-definition-label>`

    Arguments:
        property_type(str): string enum
        property_name(str): name of an Eclipse property
        time_step(int): the time step for which to get the property for
        porosity_model(str): string enum

    Returns:
        A list containing double values
        Loop through the chunks and then the values within the chunk to get all values.
    """
    all_values = []
    generator = self.selected_cell_property_async(
        property_type, property_name, time_step, porosity_model
    )
    for chunk in generator:
        for value in chunk.values:
            all_values.append(value)
    return all_values


@add_method(Case)
def grid_property_async(
    self,
    property_type,
    property_name,
    time_step,
    grid_index=0,
    porosity_model="MATRIX_MODEL",
):
    """Get a cell property for all grid cells. Async, so returns an iterator. For argument details, see :ref:`Result Definition <result-definition-label>`

    Arguments:
        property_type(str): string enum
        property_name(str): name of an Eclipse property
        time_step(int): the time step for which to get the property for
        gridIndex(int): index to the grid we're getting values for
        porosity_model(str): string enum

    Returns:
        An iterator to a chunk object containing an array of double values
        Loop through the chunks and then the values within the chunk to get all values.
    """
    property_type_enum = Properties_pb2.PropertyType.Value(property_type)
    porosity_model_enum = Case_pb2.PorosityModelType.Value(porosity_model)
    request = Properties_pb2.PropertyRequest(
        case_request=self.__request(),
        property_type=property_type_enum,
        property_name=property_name,
        time_step=time_step,
        grid_index=grid_index,
        porosity_model=porosity_model_enum,
    )
    for chunk in self.__properties_stub.GetGridProperty(request):
        yield chunk


@add_method(Case)
def grid_property(
    self,
    property_type,
    property_name,
    time_step,
    grid_index=0,
    porosity_model="MATRIX_MODEL",
):
    """Get a cell property for all grid cells. Synchronous, so returns a list. For argument details, see :ref:`Result Definition <result-definition-label>`

    Arguments:
        property_type(str): string enum
        property_name(str): name of an Eclipse property
        time_step(int): the time step for which to get the property for
        grid_index(int): index to the grid we're getting values for
        porosity_model(str): string enum

    Returns:
        A list of double values
    """
    all_values = []
    generator = self.grid_property_async(
        property_type, property_name, time_step, grid_index, porosity_model
    )
    for chunk in generator:
        for value in chunk.values:
            all_values.append(value)
    return all_values


@add_method(Case)
def set_active_cell_property_async(
    self,
    values_iterator,
    property_type,
    property_name,
    time_step,
    porosity_model="MATRIX_MODEL",
):
    """Set cell property for all active cells Async. Takes an iterator to the input values. For argument details, see :ref:`Result Definition <result-definition-label>`

    Arguments:
        values_iterator(iterator): an iterator to the properties to be set
        property_type(str): string enum
        property_name(str): name of an Eclipse property
        time_step(int): the time step for which to get the property for
        porosity_model(str): string enum
    """
    property_type_enum = Properties_pb2.PropertyType.Value(property_type)
    porosity_model_enum = Case_pb2.PorosityModelType.Value(porosity_model)
    request = Properties_pb2.PropertyRequest(
        case_request=self.__request(),
        property_type=property_type_enum,
        property_name=property_name,
        time_step=time_step,
        porosity_model=porosity_model_enum,
    )

    request_iterator = self.__generate_property_input_iterator(values_iterator, request)
    self.__properties_stub.SetActiveCellProperty(request_iterator)


@add_method(Case)
def set_active_cell_property(
    self, values, property_type, property_name, time_step, porosity_model="MATRIX_MODEL"
):
    """Set a cell property for all active cells. For argument details, see :ref:`Result Definition <result-definition-label>`

    Arguments:
        values(list): a list of double precision floating point numbers
        property_type(str): string enum
        property_name(str): name of an Eclipse property
        time_step(int): the time step for which to get the property for
        porosity_model(str): string enum
    """
    property_type_enum = Properties_pb2.PropertyType.Value(property_type)
    porosity_model_enum = Case_pb2.PorosityModelType.Value(porosity_model)
    request = Properties_pb2.PropertyRequest(
        case_request=self.__request(),
        property_type=property_type_enum,
        property_name=property_name,
        time_step=time_step,
        porosity_model=porosity_model_enum,
    )
    request_iterator = self.__generate_property_input_chunks(values, request)
    reply = self.__properties_stub.SetActiveCellProperty(request_iterator)
    if reply.accepted_value_count < len(values):
        raise IndexError


@add_method(Case)
def set_grid_property(
    self,
    values,
    property_type,
    property_name,
    time_step,
    grid_index=0,
    porosity_model="MATRIX_MODEL",
):
    """Set a cell property for all grid cells. For argument details, see :ref:`Result Definition <result-definition-label>`

    Arguments:
        values(list): a list of double precision floating point numbers
        property_type(str): string enum
        property_name(str): name of an Eclipse property
        time_step(int): the time step for which to get the property for
        grid_index(int): index to the grid we're setting values for
        porosity_model(str): string enum
    """
    property_type_enum = Properties_pb2.PropertyType.Value(property_type)
    porosity_model_enum = Case_pb2.PorosityModelType.Value(porosity_model)
    request = Properties_pb2.PropertyRequest(
        case_request=self.__request(),
        property_type=property_type_enum,
        property_name=property_name,
        time_step=time_step,
        grid_index=grid_index,
        porosity_model=porosity_model_enum,
    )
    request_iterator = self.__generate_property_input_chunks(values, request)
    reply = self.__properties_stub.SetGridProperty(request_iterator)
    if reply.accepted_value_count < len(values):
        raise IndexError


@add_method(Case)
def export_property(
    self,
    time_step,
    property_name,
    eclipse_keyword=property,
    undefined_value=0.0,
    export_file=property,
):
    """Export an Eclipse property

    Arguments:
        time_step (int): time step index
        property_name (str): property to export
        eclipse_keyword (str): Keyword used in export header. Defaults: value of property
        undefined_value (double): Value to use for undefined values. Defaults to 0.0
        export_file (str): File name for export. Defaults to the value of property parameter
    """
    return self._execute_command(
        exportProperty=Cmd.ExportPropertyRequest(
            caseId=self.id,
            timeStep=time_step,
            property=property_name,
            eclipseKeyword=eclipse_keyword,
            undefinedValue=undefined_value,
            exportFile=export_file,
        )
    )


@add_method(Case)
def create_well_bore_stability_plot(self, well_path, time_step, parameters=None):
    """Create a new well bore stability plot

    Arguments:
        well_path(str): well path name
        time_step(int): time step

    Returns:
        :class:`rips.generated.generated_classes.WellBoreStabilityPlot`
    """
    pb2_parameters = None
    if parameters is not None:
        assert isinstance(parameters, WbsParameters)
        pb2_parameters = parameters.pb2_object()

    plot_result = self._execute_command(
        createWellBoreStabilityPlot=Cmd.CreateWbsPlotRequest(
            caseId=self.id,
            wellPath=well_path,
            timeStep=time_step,
            wbsParameters=pb2_parameters,
        )
    )
    project = self.ancestor(rips.project.Project)
    plot = project.plot(view_id=plot_result.createWbsPlotResult.viewId)
    return plot


@add_method(Case)
def import_formation_names(self, formation_files=None):
    """Import formation names into project and apply it to the current case

    Arguments:
        formation_files(list): list of files to import

    """
    if formation_files is None:
        formation_files = []
    elif isinstance(formation_files, str):
        formation_files = [formation_files]

    self._execute_command(
        importFormationNames=Cmd.ImportFormationNamesRequest(
            formationFiles=formation_files, applyToCaseId=self.id
        )
    )


@add_method(Case)
def simulation_wells(self):
    """Get a list of all simulation wells for a case

    Returns:
        :class:`rips.generated.generated_classes.SimulationWell`

    """
    wells = self.descendants(SimulationWell)
    return wells


@add_method(Case)
def active_cell_centers_async(self, porosity_model="MATRIX_MODEL"):
    """Get a cell centers for all active cells. Async, so returns an iterator

    Arguments:
        porosity_model(str): string enum. See available()

    Returns:
        An iterator to a chunk object containing an array of Vec3d values.
        Loop through the chunks and then the values within the chunk to get all values.
    """
    porosity_model_enum = Case_pb2.PorosityModelType.Value(porosity_model)
    request = Case_pb2.CellInfoRequest(
        case_request=self.__request(), porosity_model=porosity_model_enum
    )
    return self.__case_stub.GetCellCenterForActiveCells(request)


@add_method(Case)
def active_cell_centers(self, porosity_model="MATRIX_MODEL"):
    """Get a cell centers for all active cells. Synchronous, so returns a list.

    Arguments:
        porosity_model(str): string enum. See available()

    Returns:
        A list of Vec3d
    """
    cell_centers = []
    generator = self.active_cell_centers_async(porosity_model)
    for chunk in generator:
        for value in chunk.centers:
            cell_centers.append(value)
    return cell_centers


@add_method(Case)
def active_cell_corners_async(self, porosity_model="MATRIX_MODEL"):
    """Get a cell corners for all active cells. Async, so returns an iterator

    Arguments:
        porosity_model(str): string enum. See available()

    Returns:
        An iterator to a chunk object containing an array of CellCorners (which is eight Vec3d values).
        Loop through the chunks and then the values within the chunk to get all values.
    """
    porosity_model_enum = Case_pb2.PorosityModelType.Value(porosity_model)
    request = Case_pb2.CellInfoRequest(
        case_request=self.__request(), porosity_model=porosity_model_enum
    )
    return self.__case_stub.GetCellCornersForActiveCells(request)


@add_method(Case)
def active_cell_corners(self, porosity_model="MATRIX_MODEL"):
    """Get a cell corners for all active cells. Synchronous, so returns a list.

        Arguments:
            porosity_model(str): string enum. See available()

    **CellCorner class description**::

        Parameter   | Description   | Type
        ----------- | ------------  | -----
        c0          |               | Vec3d
        c1          |               | Vec3d
        c2          |               | Vec3d
        c3          |               | Vec3d
        c4          |               | Vec3d
        c5          |               | Vec3d
        c6          |               | Vec3d
        c7          |               | Vec3d


    """
    cell_corners = []
    generator = self.active_cell_corners_async(porosity_model)
    for chunk in generator:
        for value in chunk.cells:
            cell_corners.append(value)
    return cell_corners


@add_method(Case)
def selected_cells_async(self):
    """Get the selected cells. Async, so returns an iterator.

    Returns:
        An iterator to a chunk object containing an array of cells.
        Loop through the chunks and then the cells within the chunk to get all cells.
    """
    return self.__case_stub.GetSelectedCells(self.__request())


@add_method(Case)
def selected_cells(self):
    """Get the selected cells. Synchronous, so returns a list.

    Returns:
        A list of Cells.
    """
    cells = []
    generator = self.selected_cells_async()
    for chunk in generator:
        for value in chunk.cells:
            cells.append(value)
    return cells


@add_method(Case)
def coarsening_info(self):
    """Get a coarsening information for all grids in the case.

    Returns:
        A list of CoarseningInfo objects with two Vec3i min and max objects
        for each entry.
    """
    return self.__case_stub.GetCoarseningInfoArray(self.__request()).data


@add_method(Case)
def available_nnc_properties(self):
    """Get a list of available NNC properties

    **NNCConnection class description**::

          Parameter               | Description                                   | Type
          ------------------------| --------------------------------------------- | -----
          cell_grid_index1        | Reservoir Cell Index to cell 1                | int32
          cell_grid_index2        | Reservoir Cell Index to cell 2                | int32
          cell1                   | Reservoir Cell IJK to cell 1                  | Vec3i
          cell2                   | Reservoir Cell IJK to cell 1                  | Vec3i

    """
    return self.__nnc_properties_stub.GetAvailableNNCProperties(
        self.__request()
    ).properties


@add_method(Case)
def nnc_connections_async(self):
    """Get the NNC connections. Async, so returns an iterator.

    Returns:
        An iterator to a chunk object containing an array NNCConnection objects.
        Loop through the chunks and then the connection within the chunk to get all connections.
    """
    return self.__nnc_properties_stub.GetNNCConnections(self.__request())


@add_method(Case)
def nnc_connections(self):
    """Get the NNC connection. Synchronous, so returns a list.

    Returns:
        A list of NNCConnection objects.
    """
    connections = []
    generator = self.nnc_connections_async()
    for chunk in generator:
        for value in chunk.connections:
            connections.append(value)
    return connections


@add_method(Case)
def __nnc_connections_values_async(self, property_name, property_type, time_step):
    request = NNCProperties_pb2.NNCValuesRequest(
        case_id=self.id,
        property_name=property_name,
        property_type=property_type,
        time_step=time_step,
    )
    return self.__nnc_properties_stub.GetNNCValues(request)


@add_method(Case)
def __nnc_values_generator_to_list(self, generator):
    """Converts a NNC values generator to a list."""
    vals = []
    for chunk in generator:
        for value in chunk.values:
            vals.append(value)
    return vals


@add_method(Case)
def nnc_connections_static_values_async(self, property_name):
    """Get the static NNC values. Async, so returns an iterator.

    Returns:
        An iterator to a chunk object containing an list of doubles.
        Loop through the chunks and then the values within the chunk to get values
        for all the connections. The order of the list matches the list from
        nnc_connections, i.e. the nth object of nnc_connections() refers to nth
        value in this list.
    """
    return self.__nnc_connections_values_async(
        property_name, NNCProperties_pb2.NNC_STATIC, 0
    )


@add_method(Case)
def nnc_connections_static_values(self, property_name):
    """Get the static NNC values.

    Returns:
        A list of doubles. The order of the list matches the list from
        nnc_connections, i.e. the nth object of nnc_connections() refers to nth
        value in this list.
    """
    generator = self.nnc_connections_static_values_async(property_name)
    return self.__nnc_values_generator_to_list(generator)


@add_method(Case)
def nnc_connections_dynamic_values_async(self, property_name, time_step):
    """Get the dynamic NNC values. Async, so returns an iterator.

    Returns:
        An iterator to a chunk object containing an list of doubles.
        Loop through the chunks and then the values within the chunk to get values
        for all the connections. The order of the list matches the list from
        nnc_connections, i.e. the nth object of nnc_connections() refers to nth
        value in this list.
    """
    return self.__nnc_connections_values_async(
        property_name, NNCProperties_pb2.NNC_DYNAMIC, time_step
    )


@add_method(Case)
def nnc_connections_dynamic_values(self, property_name, time_step):
    """Get the dynamic NNC values.

    Returns:
        A list of doubles. The order of the list matches the list from
        nnc_connections, i.e. the nth object of nnc_connections() refers to nth
        value in this list.
    """
    generator = self.nnc_connections_dynamic_values_async(property_name, time_step)
    return self.__nnc_values_generator_to_list(generator)


@add_method(Case)
def nnc_connections_generated_values_async(self, property_name, time_step):
    """Get the generated NNC values. Async, so returns an iterator.

    Returns:
        An iterator to a chunk object containing an list of doubles.
        Loop through the chunks and then the values within the chunk to get values
        for all the connections. The order of the list matches the list from
        nnc_connections, i.e. the nth object of nnc_connections() refers to nth
        value in this list.
    """
    return self.__nnc_connections_values_async(
        property_name, NNCProperties_pb2.NNC_GENERATED, time_step
    )


@add_method(Case)
def nnc_connections_generated_values(self, property_name, time_step):
    """Get the generated NNC values.

    Returns:
        A list of doubles. The order of the list matches the list from
        nnc_connections, i.e. the nth object of nnc_connections() refers to nth
        value in this list.
    """
    generator = self.nnc_connections_generated_values_async(property_name, time_step)
    return self.__nnc_values_generator_to_list(generator)


@add_method(Case)
def __generate_nnc_property_input_chunks(self, array, parameters):
    index = -1
    while index < len(array):
        chunk = NNCProperties_pb2.NNCValuesChunk()
        if index == -1:
            chunk.params.CopyFrom(parameters)
            index += 1
        else:
            actual_chunk_size = min(len(array) - index + 1, self.chunk_size)
            chunk.values.CopyFrom(
                NNCProperties_pb2.NNCValues(
                    values=array[index : index + actual_chunk_size]
                )
            )
            index += actual_chunk_size

        yield chunk
    # Final empty message to signal completion
    chunk = NNCProperties_pb2.NNCValuesChunk()
    yield chunk


@add_method(Case)
def set_nnc_connections_values(
    self, values, property_name, time_step, porosity_model="MATRIX_MODEL"
):
    """Set nnc connection values for all connections..

    Arguments:
        values(list): a list of double precision floating point numbers
        property_name(str): name of an Eclipse property
        time_step(int): the time step for which to get the property for
        porosity_model(str): string enum. See available()
    """
    porosity_model_enum = Case_pb2.PorosityModelType.Value(porosity_model)
    request = NNCProperties_pb2.NNCValuesInputRequest(
        case_id=self.id,
        property_name=property_name,
        time_step=time_step,
        porosity_model=porosity_model_enum,
    )
    request_iterator = self.__generate_nnc_property_input_chunks(values, request)
    reply = self.__nnc_properties_stub.SetNNCValues(request_iterator)
    if reply.accepted_value_count < len(values):
        raise IndexError



import sys
import os
import math
import pytest
import grpc
import tempfile

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips

import dataroot


def test_Launch(rips_instance, initialize_test):
    assert rips_instance is not None


def test_EmptyProject(rips_instance, initialize_test):
    cases = rips_instance.project.cases()
    assert len(cases) is 0


def test_OneCase(rips_instance, initialize_test):
    case = rips_instance.project.load_case(
        dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    )
    assert case.name == "TEST10K_FLT_LGR_NNC"
    assert case.id == 0
    cases = rips_instance.project.cases()
    assert len(cases) is 1


def test_BoundingBox(rips_instance, initialize_test):
    case = rips_instance.project.load_case(
        dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    )
    assert case.name == "TEST10K_FLT_LGR_NNC"
    boundingbox = case.reservoir_boundingbox()
    assert math.isclose(3382.90, boundingbox.min_x, abs_tol=1.0e-1)
    assert math.isclose(5850.48, boundingbox.max_x, abs_tol=1.0e-1)
    assert math.isclose(4157.45, boundingbox.min_y, abs_tol=1.0e-1)
    assert math.isclose(7354.93, boundingbox.max_y, abs_tol=1.0e-1)
    assert math.isclose(-4252.61, boundingbox.min_z, abs_tol=1.0e-1)
    assert math.isclose(-4103.60, boundingbox.max_z, abs_tol=1.0e-1)

    min_depth, max_depth = case.reservoir_depth_range()
    assert math.isclose(4103.60, min_depth, abs_tol=1.0e-1)
    assert math.isclose(4252.61, max_depth, abs_tol=1.0e-1)


def test_MultipleCases(rips_instance, initialize_test):
    case_paths = []
    case_paths.append(dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID")
    case_paths.append(dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID")
    case_paths.append(dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID")

    case_names = []
    for case_path in case_paths:
        case_name = os.path.splitext(os.path.basename(case_path))[0]
        case_names.append(case_name)
        rips_instance.project.load_case(path=case_path)

    cases = rips_instance.project.cases()
    assert len(cases) == len(case_names)
    for i, case_name in enumerate(case_names):
        assert case_name == cases[i].name


def get_cell_index_with_ijk(cell_info, i, j, k):
    for idx, cell in enumerate(cell_info):
        if cell.local_ijk.i == i and cell.local_ijk.j == j and cell.local_ijk.k == k:
            return idx
    return -1


def check_corner(actual, expected):
    assert math.isclose(actual.x, expected[0], abs_tol=0.1)
    assert math.isclose(actual.y, expected[1], abs_tol=0.1)
    assert math.isclose(actual.z, expected[2], abs_tol=0.1)


def test_10k(rips_instance, initialize_test):
    case_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=case_path)
    assert len(case.grids()) == 2
    cell_count_info = case.cell_count()
    assert cell_count_info.active_cell_count == 11125
    assert cell_count_info.reservoir_cell_count == 316224
    time_steps = case.time_steps()
    assert len(time_steps) == 9
    days_since_start = case.days_since_start()
    assert len(days_since_start) == 9
    cell_info = case.cell_info_for_active_cells()
    assert len(cell_info) == cell_count_info.active_cell_count

    # Check an active cell (found in resinsight ui)
    cell_index = get_cell_index_with_ijk(cell_info, 23, 44, 19)
    assert cell_index != -1

    cell_centers = case.active_cell_centers()
    assert len(cell_centers) == cell_count_info.active_cell_count

    # Check the cell center for the specific cell
    assert math.isclose(3627.17, cell_centers[cell_index].x, abs_tol=0.1)
    assert math.isclose(5209.75, cell_centers[cell_index].y, abs_tol=0.1)
    assert math.isclose(4179.6, cell_centers[cell_index].z, abs_tol=0.1)

    cell_corners = case.active_cell_corners()
    assert len(cell_corners) == cell_count_info.active_cell_count
    # Expected values from ResInsight UI
    expected_corners = [
        [3565.22, 5179.02, 4177.18],
        [3655.67, 5145.34, 4176.63],
        [3690.07, 5240.69, 4180.02],
        [3599.87, 5275.16, 4179.32],
        [3564.13, 5178.61, 4179.75],
        [3654.78, 5144.79, 4179.23],
        [3688.99, 5239.88, 4182.7],
        [3598.62, 5274.48, 4181.96],
    ]
    check_corner(cell_corners[cell_index].c0, expected_corners[0])
    check_corner(cell_corners[cell_index].c1, expected_corners[1])
    check_corner(cell_corners[cell_index].c2, expected_corners[2])
    check_corner(cell_corners[cell_index].c3, expected_corners[3])
    check_corner(cell_corners[cell_index].c4, expected_corners[4])
    check_corner(cell_corners[cell_index].c5, expected_corners[5])
    check_corner(cell_corners[cell_index].c6, expected_corners[6])
    check_corner(cell_corners[cell_index].c7, expected_corners[7])

    # No coarsening info for this case
    coarsening_info = case.coarsening_info()
    assert len(coarsening_info) == 0


def test_PdmObject(rips_instance, initialize_test):
    case_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=case_path)
    assert case.id == 0
    assert case.address() is not 0
    assert case.__class__.__name__ == "EclipseCase"


@pytest.mark.skipif(
    sys.platform.startswith("linux"),
    reason="Brugge is currently exceptionally slow on Linux",
)
def test_brugge_0010(rips_instance, initialize_test):
    case_path = dataroot.PATH + "/Case_with_10_timesteps/Real10/BRUGGE_0010.EGRID"
    case = rips_instance.project.load_case(path=case_path)
    assert len(case.grids()) == 1
    cellCountInfo = case.cell_count()
    assert cellCountInfo.active_cell_count == 43374
    assert cellCountInfo.reservoir_cell_count == 60048
    time_steps = case.time_steps()
    assert len(time_steps) == 11
    days_since_start = case.days_since_start()
    assert len(days_since_start) == 11


@pytest.mark.skipif(
    sys.platform.startswith("linux"),
    reason="Brugge is currently exceptionally slow on Linux",
)
def test_replaceCase(rips_instance, initialize_test):
    project = rips_instance.project.open(
        dataroot.PATH + "/TEST10K_FLT_LGR_NNC/10KWithWellLog.rsp"
    )
    case_path = dataroot.PATH + "/Case_with_10_timesteps/Real0/BRUGGE_0000.EGRID"
    case = project.case(case_id=0)
    assert case is not None
    assert case.name == "TEST10K_FLT_LGR_NNC"
    assert case.id == 0
    cases = rips_instance.project.cases()
    assert len(cases) is 1

    case.replace(new_grid_file=case_path)
    # Check that the case object has been changed
    assert case.name == "BRUGGE_0000"
    assert case.id == 0

    cases = rips_instance.project.cases()
    assert len(cases) is 1
    # Check that retrieving the case object again will yield the changed object
    case = project.case(case_id=0)
    assert case.name == "BRUGGE_0000"
    assert case.id == 0


def test_loadNonExistingCase(rips_instance, initialize_test):
    case_path = "Nonsense/Nonsense/Nonsense"
    with pytest.raises(grpc.RpcError):
        assert rips_instance.project.load_case(case_path)


@pytest.mark.skipif(
    sys.platform.startswith("linux"),
    reason="Brugge is currently exceptionally slow on Linux",
)
def test_exportFlowCharacteristics(rips_instance, initialize_test):
    case_path = dataroot.PATH + "/Case_with_10_timesteps/Real0/BRUGGE_0000.EGRID"
    case = rips_instance.project.load_case(case_path)
    with tempfile.TemporaryDirectory(prefix="rips") as tmpdirname:
        print("Temporary folder: ", tmpdirname)
        file_name = tmpdirname + "/exportFlowChar.txt"
        case.export_flow_characteristics(
            time_steps=8, producers=[], injectors="I01", file_name=file_name
        )


def test_selected_cells(rips_instance, initialize_test):
    case = rips_instance.project.load_case(
        dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    )
    assert case.name == "TEST10K_FLT_LGR_NNC"
    selected_cells = case.selected_cells()
    assert len(selected_cells) == 0

    time_step_info = case.time_steps()
    for tidx, timestep in enumerate(time_step_info):
        # Try to read for SOIL the time step (will be empty since nothing is selected)
        soil_results = case.selected_cell_property("DYNAMIC_NATIVE", "SOIL", tidx)
        assert len(soil_results) == 0


def test_multiple_load_of_same_case(rips_instance, initialize_test):
    # Test related to issue https://github.com/OPM/ResInsight/issues/9332
    path_name = dataroot.PATH + "/flow_diagnostics_test/SIMPLE_SUMMARY2.EGRID"
    case_count = 3
    for i in range(case_count):
        case = rips_instance.project.load_case(path_name)



import sys
import os

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips

import dataroot


def test_10k(rips_instance, initialize_test):
    case_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=case_path)
    case.create_view()
    assert len(case.grids()) == 2
    cell_count_info = case.cell_count()

    sim_wells = case.simulation_wells()
    assert len(sim_wells) == 3

    assert sim_wells[0].name == "GI1"
    assert sim_wells[1].name == "GP1"
    assert sim_wells[2].name == "GP2"

    timesteps = case.time_steps()

    # On time step 0 all simulation wells are undefined
    for sim_well in sim_wells:
        status = sim_well.status(0)
        assert status.well_type == "NotDefined"

    # On time step 3 all wells are producing
    for sim_well in sim_wells:
        status = sim_well.status(3)
        assert status.well_type == "Producer"

    # On time step 0 all simulation wells have no cells
    for sim_well in sim_wells:
        cells = sim_well.cells(0)
        assert len(cells) == 0

    # On the other time steps there should be cells
    expected_cell_count = {}
    expected_cell_count["GP1"] = 105
    expected_cell_count["GI1"] = 38
    expected_cell_count["GP2"] = 18
    for sim_well in sim_wells:
        for tidx, timestep in enumerate(timesteps):
            if tidx > 0:
                cells = sim_well.cells(tidx)
                print(
                    "well: "
                    + sim_well.name
                    + " timestep: "
                    + str(tidx)
                    + " cells:"
                    + str(len(cells))
                )
                assert len(cells) == expected_cell_count[sim_well.name]



import rips
import time

resinsight = rips.Instance.find()

project = resinsight.project

# Use the following commented lines to import a file from disk
# filename = "path/to/file/1_R001_REEK-0.SMSPEC"
# summary_case = project.import_summary_case(filename)

# Assumes at least one summery case loaded with case_id 1
summary_case = project.summary_case(1)
if summary_case is None:
    print("No summary case found")
    exit()

vector_name = "FOPT"
summary_data = summary_case.summary_vector_values(vector_name)

print("Data for summary vector " + vector_name)
print(summary_data.values)

time_steps = summary_case.available_time_steps()
print(time_steps.values)

summary_data_sampled = summary_case.resample_values("FOPT", "QUARTER")
print("\nResampled data")

for t, value in zip(summary_data_sampled.time_steps, summary_data_sampled.values):
    print(time.strftime("%a, %d %b %Y ", time.gmtime(t)) + " | " + str(value))

test_values = summary_data.values
offset = test_values[len(test_values) - 1] / 10

for index, item in enumerate(test_values):
    test_values[index] = test_values[index] + offset

summary_case.set_summary_values("FOPT_M1", "myUnit", test_values)

for index, item in enumerate(test_values):
    test_values[index] = test_values[index] + offset
summary_case.set_summary_values("FOPT_M2", "myUnit", test_values)

for index, item in enumerate(test_values):
    test_values[index] = test_values[index] + offset
summary_case.set_summary_values("FOPT_M3", "myUnit", test_values)

for index, item in enumerate(test_values):
    test_values[index] = test_values[index] + offset
summary_case.set_summary_values("FOPT_M4", "myUnit", test_values)



import sys
import os
import math
import contextlib
import os
import shutil
import tempfile

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips

import dataroot


def test_summary_import_and_find(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/flow_diagnostics_test/SIMPLE_SUMMARY2.SMSPEC"
    summary_case = rips_instance.project.import_summary_case(casePath)
    assert summary_case.id == 1

    case_id = 234
    found_summary_case = rips_instance.project.summary_case(case_id)
    assert found_summary_case is None

    correct_case_id = 1
    found_summary_case = rips_instance.project.summary_case(correct_case_id)
    assert found_summary_case is not None

    rips_instance.project.close()
    correct_case_id = 1
    found_summary_case = rips_instance.project.summary_case(correct_case_id)
    assert found_summary_case is None


def test_summary_data(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/flow_diagnostics_test/SIMPLE_SUMMARY2.SMSPEC"
    summary_case = rips_instance.project.import_summary_case(casePath)
    assert summary_case.id == 1

    addresses = summary_case.available_addresses()
    # Summary reader type is controlled from Preferences. libecl reports 343 vectors, opm_common (ESMRY) reports 339.
    # As this configuration can be different, allow both variants
    assert len(addresses.values) == 335 or len(addresses.values) == 339

    summary_data = summary_case.summary_vector_values("FOPT")
    assert len(summary_data.values) == 60


def test_summary_resample(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/flow_diagnostics_test/SIMPLE_SUMMARY2.SMSPEC"
    summary_case = rips_instance.project.import_summary_case(casePath)
    assert summary_case.id == 1

    summary_data_sampled = summary_case.resample_values("FOPT", "NONE")
    assert len(summary_data_sampled.values) == 60
    assert len(summary_data_sampled.time_steps) == 60

    summary_data_sampled = summary_case.resample_values("FOPT", "DAY")
    assert len(summary_data_sampled.values) == 721
    assert len(summary_data_sampled.time_steps) == 721

    summary_data_sampled = summary_case.resample_values("FOPT", "MONTH")
    assert len(summary_data_sampled.values) == 24
    assert len(summary_data_sampled.time_steps) == 24

    summary_data_sampled = summary_case.resample_values("FOPT", "QUARTER")
    assert len(summary_data_sampled.values) == 8
    assert len(summary_data_sampled.time_steps) == 8

    summary_data_sampled = summary_case.resample_values("FOPT", "YEAR")
    assert len(summary_data_sampled.values) == 3
    assert len(summary_data_sampled.time_steps) == 3


@contextlib.contextmanager
def cd(newdir, cleanup=lambda: True):
    prevdir = os.getcwd()
    os.chdir(os.path.expanduser(newdir))
    try:
        yield
    finally:
        os.chdir(prevdir)
        cleanup()


@contextlib.contextmanager
def tempdir():
    dirpath = tempfile.mkdtemp()

    def cleanup():
        shutil.rmtree(dirpath)

    with cd(dirpath, cleanup):
        yield dirpath


def test_summary_set_values(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/flow_diagnostics_test/SIMPLE_SUMMARY2.SMSPEC"
    summary_case = rips_instance.project.import_summary_case(casePath)
    assert summary_case.id == 1

    addresses = summary_case.available_addresses()
    original_keyword_count = len(addresses.values)

    summary_data = summary_case.summary_vector_values("FOPT")
    assert len(summary_data.values) == 60

    summary_case.set_summary_values("FOPT_1", "", summary_data.values)
    generated_summary_data = summary_case.summary_vector_values("FOPT_1")
    assert len(generated_summary_data.values) == 60

    addresses = summary_case.available_addresses()
    current_keyword_count = len(addresses.values)
    assert current_keyword_count == original_keyword_count + 1

    # Using existing keyword will overwrite existing data
    summary_case.set_summary_values("FOPT_1", "", summary_data.values)
    addresses = summary_case.available_addresses()
    current_keyword_count = len(addresses.values)
    assert current_keyword_count == original_keyword_count + 1

    # invalid value count, check that available addresses are unchanged
    summary_case.set_summary_values("FOPT_2", "", [])
    addresses = summary_case.available_addresses()
    current_keyword_count = len(addresses.values)
    assert current_keyword_count == original_keyword_count + 1



# Load ResInsight Processing Server Client Library
import rips
import tempfile
from os.path import expanduser
from pathlib import Path

# Connect to ResInsight instance
resinsight = rips.Instance.find()
# Example code
project = resinsight.project

# Look for input files in the home directory of the user
home_dir = expanduser("~")
elastic_properties_file_path = (Path(home_dir) / "elastic_properties.csv").as_posix()
print("Elastic properties file path:", elastic_properties_file_path)

facies_properties_file_path = (Path(home_dir) / "facies_id.roff").as_posix()
print("Facies properties file path:", facies_properties_file_path)

# Find a case
cases = resinsight.project.cases()
case = cases[1]

# Use the last time step
time_steps = case.time_steps()
time_step = time_steps[len(time_steps) - 1]


# Create stim plan model template
fmt_collection = project.descendants(rips.StimPlanModelTemplateCollection)[0]
stim_plan_model_template = fmt_collection.append_stim_plan_model_template(
    eclipse_case=case,
    time_step=time_step,
    elastic_properties_file_path=elastic_properties_file_path,
    facies_properties_file_path=facies_properties_file_path,
)
stim_plan_model_template.overburden_formation = "Garn"
stim_plan_model_template.overburden_facies = "Shale"
stim_plan_model_template.underburden_formation = "Garn"
stim_plan_model_template.underburden_facies = "Shale"
stim_plan_model_template.overburden_height = 68
stim_plan_model_template.update()
print("Overburden: ", stim_plan_model_template.overburden_formation)


# Set eclipse result for facies definition
eclipse_result = stim_plan_model_template.facies_properties().facies_definition()
eclipse_result.result_type = "INPUT_PROPERTY"
eclipse_result.result_variable = "OPERNUM_1"
eclipse_result.update()

# Set eclipse result for non-net layers
non_net_layers = stim_plan_model_template.non_net_layers()
non_net_layers_result = non_net_layers.facies_definition()
non_net_layers_result.result_type = "STATIC_NATIVE"
non_net_layers_result.result_variable = "NTG"
non_net_layers_result.update()
non_net_layers.formation = "Not"
non_net_layers.facies = "Shale"
non_net_layers.update()


# Add some scaling factors
elastic_properties = stim_plan_model_template.elastic_properties()
elastic_properties.add_property_scaling(
    formation="Garn", facies="Calcite", property="YOUNGS_MODULUS", scale=1.44
)


well_name = "B-2 H"

# Find a well
well_path = project.well_path_by_name(well_name)
print("well path:", well_path)
stim_plan_model_collection = project.descendants(rips.StimPlanModelCollection)[0]


export_folder = tempfile.gettempdir()

stim_plan_models = []

# Create and export a StimPlan model for each depth
measured_depths = [3200.0, 3400.0, 3600.0]
for measured_depth in measured_depths:
    # Create stim plan model at a give measured depth
    stim_plan_model = stim_plan_model_collection.append_stim_plan_model(
        well_path=well_path,
        measured_depth=measured_depth,
        stim_plan_model_template=stim_plan_model_template,
    )
    stim_plan_models.append(stim_plan_model)

    # Make the well name safer to use as a directory path
    well_name_part = well_name.replace(" ", "_")
    directory_path = Path(export_folder) / "{}_{}".format(
        well_name_part, int(measured_depth)
    )

    # Create the folder
    directory_path.mkdir(parents=True, exist_ok=True)

    print("Exporting fracture model to: ", directory_path)
    stim_plan_model.export_to_file(directory_path=directory_path.as_posix())

    # Create a fracture mode plot
    stim_plan_model_plot_collection = project.descendants(
        rips.StimPlanModelPlotCollection
    )[0]
    stim_plan_model_plot = stim_plan_model_plot_collection.append_stim_plan_model_plot(
        stim_plan_model=stim_plan_model
    )

    print("Exporting fracture model plot to: ", directory_path)
    stim_plan_model_plot.export_snapshot(export_folder=directory_path.as_posix())


print("Setting measured depth and perforation length.")
stim_plan_models[0].measured_depth = 3300.0
stim_plan_models[0].perforation_length = 123.445
stim_plan_models[0].update()



"""
ResInsight 2d plot module
"""
import Commands_pb2

from .pdmobject import add_method
from .resinsight_classes import PlotWindow, Plot


@add_method(PlotWindow)
def export_snapshot(self, export_folder="", file_prefix="", output_format="PNG"):
    """Export snapshot for the current plot

    Arguments:
        export_folder(str): The path to export to. By default will use the global export folder
        prefix (str): Exported file name prefix
        output_format(str): Enum string. Can be 'PNG' or 'PDF'.

    """
    return self._execute_command(
        exportSnapshots=Commands_pb2.ExportSnapshotsRequest(
            type="PLOTS",
            prefix=file_prefix,
            viewId=self.id,
            exportFolder=export_folder,
            plotOutputFormat=output_format,
        )
    )



###################################################################################
# This example will connect to ResInsight, retrieve a list of
# simulation wells and print info
###################################################################################

# Import the ResInsight Processing Server Module
import rips

# Connect to ResInsight
resinsight = rips.Instance.find()
if resinsight is not None:
    # Get a list of all wells
    cases = resinsight.project.cases()

    for case in cases:
        print("Case id: " + str(case.id))
        print("Case name: " + case.name)

        timesteps = case.time_steps()
        sim_wells = case.simulation_wells()
        for sim_well in sim_wells:
            print("Simulation well: " + sim_well.name)

            for tidx, timestep in enumerate(timesteps):
                status = sim_well.status(tidx)
                cells = sim_well.cells(tidx)
                print(
                    "timestep: "
                    + str(tidx)
                    + " type: "
                    + status.well_type
                    + " open: "
                    + str(status.is_open)
                    + " cells:"
                    + str(len(cells))
                )



######################################################################
# This script applies a flow diagnostics cell result to the first view in the project
######################################################################

# Load ResInsight Processing Server Client Library
import rips

# Connect to ResInsight instance
resinsight = rips.Instance.find()

view = resinsight.project.view(view_id=1)
# view.apply_flow_diagnostics_cell_result(result_variable='Fraction',
#                                    selection_mode='FLOW_TR_INJ_AND_PROD')

# Example of setting individual wells. Commented out because well names are case specific.
view.apply_flow_diagnostics_cell_result(
    result_variable="Fraction",
    selection_mode="FLOW_TR_BY_SELECTION",
    injectors=["C-1H", "C-2H", "F-2H"],
    producers=["B-1AH", "B-3H", "D-1H"],
)
